"""File formats.

This module implements the read(), iread() and write() functions in ase.io.
For each file format there is a namedtuple (IOFormat) that has the following
elements:

* a read(filename, index, **kwargs) generator that will yield Atoms objects
* a write(filename, images) function
* a 'single' boolean (False if multiple configurations is supported)
* a 'acceptsfd' boolean (True if file-descriptors are accepted)

There is a dict 'ioformats' that is filled with IOFormat objects as they are
needed.  The 'initialize()' function will create the IOFormat object by
looking at the all_formats dict and by importing the correct read/write
functions from the correct module.  The 'single' and 'acceptsfd' bools are
parsed from two-charcter string in the all_formats dict below.


Example
=======

The xyz format is implemented in the ase/io/xyz.py file which has a
read_xyz() generator and a write_xyz() function.

"""

import collections
import functools
import inspect
import os
import sys

from ase.atoms import Atoms
from ase.utils import import_module, basestring
from ase.parallel import parallel_function, parallel_generator

IOFormat = collections.namedtuple('IOFormat', 'read, write, single, acceptsfd')
ioformats = {}  # will be filled at run-time

# 1=single, +=multiple, F=accepts a file-descriptor, S=needs a file-name str
all_formats = {
    'abinit': ('ABINIT input file', '1F'),
    'aims': ('FHI-aims geometry file', '1S'),
    'aims-output': ('FHI-aims output', '+S'),
    'bundletrajectory': ('ASE bundle trajectory', '+S'),
    'castep-castep': ('CASTEP output file', '+F'),
    'castep-cell': ('CASTEP geom file', '1F'),
    'castep-geom': ('CASTEP trajectory file', '+F'),
    'castep-md': ('CASTEP molecular dynamics file', '+F'),
    'castep-phonon': ('CASTEP phonon file', '1F'),
    'cfg': ('AtomEye configuration', '1F'),
    'cif': ('CIF-file', '+F'),
    'cmdft': ('CMDFT-file', '1F'),
    'cube': ('CUBE file', '1F'),
    'dacapo': ('Dacapo netCDF output file', '1F'),
    'dacapo-text': ('Dacapo text output', '1F'),
    'db': ('ASE SQLite database file', '+S'),
    'dftb': ('DftbPlus input file', '1S'),
    'elk': ('ELK atoms definition', '1S'),
    'eon': ('EON reactant.con file', '1F'),
    'eps': ('Encapsulated Postscript', '1S'),
    'espresso-in': ('Quantum espresso in file', '1F'),
    'espresso-out': ('Quantum espresso out file', '1F'),
    'etsf': ('ETSF format', '1S'),
    'exciting': ('exciting input', '1S'),
    'extxyz': ('Extended XYZ file', '+F'),
    'findsym': ('FINDSYM-format', '+F'),
    'gaussian': ('Gaussian com (input) file', '1S'),
    'gaussian-out': ('Gaussian output file', '1F'),
    'gen': ('DFTBPlus GEN format', '1F'),
    'gpaw-out': ('GPAW text output', '+F'),
    'gpw': ('GPAW restart-file', '1S'),
    'gromacs': ('Gromacs coordinates', '1S'),
    'gromos': ('Gromos96 geometry file', '1F'),
    'html': ('X3DOM HTML', '1S'),
    'iwm': ('?', '1F'),
    'json': ('ASE JSON database file', '+S'),
    'jsv': ('JSV file format', '1F'),
    'lammps-dump': ('LAMMPS dump file', '1F'),
    'lammps-data': ('LAMMPS data file', '1F'),
    'magres': ('MAGRES ab initio NMR data file', '1S'),
    'mol': ('MDL Molfile', '1F'),
    'nwchem': ('NWChem input file', '1F'),
    'octopus': ('Octopus input file', '1F'),
    'proteindatabank': ('Protein Data Bank', '+F'),
    'png': ('Portable Network Graphics', '1S'),
    'postgresql': ('ASE PostgreSQL database file', '+S'),
    'pov': ('Persistance of Vision', '1S'),
    'py': ('Python file', '+F'),
    'res': ('SHELX format', '1S'),
    'sdf': ('SDF format', '1F'),
    'struct': ('WIEN2k structure file', '1S'),
    'struct_out': ('SIESTA STRUCT file', '1F'),
    'traj': ('ASE trajectory', '+S'),
    'trj': ('Old ASE pickle trajectory', '+S'),
    'turbomole': ('TURBOMOLE coord file', '1F'),
    'turbomole-gradient': ('TURBOMOLE gradient file', '+F'),
    'v-sim': ('V_Sim ascii file', '1F'),
    'vasp': ('VASP POSCAR/CONTCAR file', '1F'),
    'vasp-out': ('VASP OUTCAR file', '+F'),
    'vasp-xdatcar': ('VASP XDATCAR file', '+S'),
    'vasp-xml': ('VASP vasprun.xml file', '+F'),
    'vti': ('VTK XML Image Data', '1F'),
    'vtu': ('VTK XML Unstructured Grid', '1F'),
    'x3d': ('X3D', '1S'),
    'xsd': ('Materials Studio file', '1F'),
    'xsf': ('XCrySDen Structure File', '+F'),
    'xyz': ('XYZ-file', '+F')}

# Special cases:
format2modulename = {
    'aims-output': 'aims',
    'castep-castep': 'castep',
    'castep-cell': 'castep',
    'castep-geom': 'castep',
    'castep-md': 'castep',
    'castep-phonon': 'castep',
    'dacapo-text': 'dacapo',
    'espresso-in': 'espresso',
    'espresso-out': 'espresso',
    'gaussian-out': 'gaussian',
    'html': 'x3d',
    'json': 'db',
    'lammps-dump': 'lammpsrun',
    'lammps-data': 'lammpsdata',
    'postgresql': 'db',
    'struct': 'wien2k',
    'struct_out': 'siesta',
    'traj': 'trajectory',
    'trj': 'pickletrajectory',
    'turbomole-gradient': 'turbomole',
    'vasp-out': 'vasp',
    'vasp-xdatcar': 'vasp',
    'vasp-xml': 'vasp',
    'vti': 'vtkxml',
    'vtu': 'vtkxml'}

extension2format = {
    'ascii': 'v-sim',
    'castep': 'castep-castep',
    'cell': 'castep-cell',
    'com': 'gaussian',
    'con': 'eon',
    'exi': 'exciting',
    'g96': 'gromos',
    'geom': 'castep-geom',
    'gro': 'gromacs',
    'log': 'gaussian-out',
    'md': 'castep-md',
    'nw': 'nwchem',
    'out': 'espresso-out',
    'pdb': 'proteindatabank',
    'shelx': 'res',
    'in': 'aims',
    'poscar': 'vasp',
    'phonon': 'castep-phonon'}


def initialize(format):
    """Import read and write functions."""
    if format in ioformats:
        return  # already done

    _format = format.replace('-', '_')
    module_name = format2modulename.get(format, _format)
    try:
        module = import_module('ase.io.' + module_name)
    except ImportError as err:
        raise ValueError('File format not recognized: %s.  Error: %s'
                         % (format, err))

    read = getattr(module, 'read_' + _format, None)
    write = getattr(module, 'write_' + _format, None)

    if read and not inspect.isgeneratorfunction(read):
        read = functools.partial(wrap_read_function, read)
    if not read and not write:
        raise ValueError('File format not recognized: ' + format)
    code = all_formats[format][1]
    single = code[0] == '1'
    acceptsfd = code[1] == 'F'
    ioformats[format] = IOFormat(read, write, single, acceptsfd)


def get_ioformat(format):
    """Initialize and return IOFormat tuple."""
    initialize(format)
    return ioformats[format]


def wrap_read_function(read, filename, index=None, **kwargs):
    """Convert read-function to generator."""
    if index is None:
        yield read(filename, **kwargs)
    else:
        for atoms in read(filename, index, **kwargs):
            yield atoms


def write(filename, images, format=None, **kwargs):
    """Write Atoms object(s) to file.

    filename: str or file
        Name of the file to write to or a file descriptor.  The name '-'
        means standard output.
    images: Atoms object or list of Atoms objects
        A single Atoms object or a list of Atoms objects.
    format: str
        Used to specify the file-format.  If not given, the
        file-format will be taken from suffix of the filename.

    The use of additional keywords is format specific."""

    if isinstance(filename, basestring):
        filename = os.path.expanduser(filename)
        fd = None
        if filename == '-':
            fd = sys.stdout
            filename = None
        elif format is None:
            format = filetype(filename, read=False)
    else:
        fd = filename
        filename = None

    format = format or 'json'  # default is json

    io = get_ioformat(format)

    _write(filename, fd, format, io, images, **kwargs)


@parallel_function
def _write(filename, fd, format, io, images, **kwargs):
    if isinstance(images, Atoms):
        images = [images]

    if io.single:
        if len(images) > 1:
            raise ValueError('{0}-format can only store 1 Atoms object.'
                             .format(format))
        images = images[0]

    if io.write is None:
        raise ValueError("Can't write to {0}-format".format(format))

    # Special case for json-format:
    if format == 'json' and len(images) > 1:
        if filename is not None:
            io.write(filename, images, **kwargs)
            return
        raise ValueError("Can't write more than one image to file-descriptor"
                         'using json-format.')

    if io.acceptsfd:
        open_new = (fd is None)
        if open_new:
            fd = open(filename, 'w')
        io.write(fd, images, **kwargs)
        if open_new:
            fd.close()
    else:
        if fd is not None:
            raise ValueError("Can't write {0}-format to file-descriptor"
                             .format(format))
        io.write(filename, images, **kwargs)


def read(filename, index=None, format=None, **kwargs):
    """Read Atoms object(s) from file.

    filename: str or file
        Name of the file to read from or a file descriptor.
    index: int, slice or str
        The last configuration will be returned by default.  Examples:

            * ``index=0``: first configuration
            * ``index=-2``: second to last
            * ``index=':'`` or ``index=slice(None)``: all
            * ``index='-3:`` or ``index=slice(-3, None)``: three last
            * ``index='::2`` or ``index=slice(0, None, 2)``: even
            * ``index='1::2`` or ``index=slice(1, None, 2)``: odd
    format: str
        Used to specify the file-format.  If not given, the
        file-format will be guessed by the *filetype* function.

    Many formats allow on open file-like object to be passed instead
    of ``filename``. In this case the format cannot be auto-decected,
    so the ``format`` argument should be explicitly given."""

    if isinstance(index, basestring):
        index = string2index(index)
    filename, index = parse_filename(filename, index)
    if index is None:
        index = -1
    format = format or filetype(filename)
    io = get_ioformat(format)
    if isinstance(index, (slice, basestring)):
        return list(_iread(filename, index, format, io, **kwargs))
    else:
        return next(_iread(filename, slice(index, None), format, io, **kwargs))


def iread(filename, index=None, format=None, **kwargs):
    """Iterator for reading Atoms objects from file.

    Works as the `read` function, but yields one Atoms object at a time
    instead of all at once."""

    if isinstance(index, basestring):
        index = string2index(index)

    filename, index = parse_filename(filename, index)

    if index is None or index == ':':
        index = slice(None, None, None)

    if not isinstance(index, (slice, basestring)):
        index = slice(index, (index + 1) or None)

    format = format or filetype(filename)
    io = get_ioformat(format)

    for atoms in _iread(filename, index, format, io, **kwargs):
        yield atoms


@parallel_generator
def _iread(filename, index, format, io, full_output=False, **kwargs):
    compression = None
    if isinstance(filename, basestring):
        filename = os.path.expanduser(filename)
        if filename.endswith('.gz'):
            compression = 'gz'
            filename = filename[:-3]
        elif filename.endswith('.bz2'):
            compression = 'bz2'
            filename = filename[:-4]

    if not io.read:
        raise ValueError("Can't read from {0}-format".format(format))

    if io.single:
        start = index.start
        assert start is None or start == 0 or start == -1
        args = ()
    else:
        args = (index,)

    must_close_fd = False
    if isinstance(filename, basestring):
        if io.acceptsfd:
            if compression == 'gz':
                import gzip
                fd = gzip.open(filename + '.gz')
            elif compression == 'bz2':
                import bz2
                fd = bz2.BZ2File(filename + '.bz2')
            else:
                fd = open(filename, 'rU')
            must_close_fd = True
        else:
            fd = filename
    else:
        assert io.acceptsfd
        fd = filename

    # Make sure fd is closed in case loop doesn't finish:
    try:
        for dct in io.read(fd, *args, **kwargs):
            if not isinstance(dct, dict):
                dct = {'atoms': dct}
            if full_output:
                yield dct
            else:
                yield dct['atoms']
    finally:
        if must_close_fd:
            fd.close()


def parse_filename(filename, index=None):
    if not isinstance(filename, basestring) or '@' not in filename:
        return filename, index
    newindex = None
    if ('.json@' in filename or
        '.db@' in filename or
        filename.startswith('pg://')):
        newfilename, newindex = filename.rsplit('@', 1)
    else:
        newfilename, newindex = filename.rsplit('@', 1)
        try:
            newindex = string2index(newindex)
        except ValueError:
            return filename, index
    return newfilename, newindex


def string2index(string):
    if ':' not in string:
        return int(string)
    i = []
    for s in string.split(':'):
        if s == '':
            i.append(None)
        else:
            i.append(int(s))
    i += (3 - len(i)) * [None]
    return slice(*i)


def filetype(filename, read=True):
    """Try to guess the type of the file.

    First, special signatures in the filename will be checked for.  If that
    does not identify the file type, then the first 2000 bytes of the file
    will be read and analysed.  Turn off this second part by using
    read=False.

    Can be used from the command-line also::

        $ python -m ase.io.formats filename ...
    """

    ext = None
    if isinstance(filename, basestring):
        if os.path.isdir(filename):
            if os.path.basename(os.path.normpath(filename)) == 'states':
                return 'eon'
            return 'bundletrajectory'

        if filename.startswith('pg://'):
            return 'postgresql'

        basename = os.path.basename(filename)

        if basename == 'inp':
            return 'octopus'

        if '.' in basename:
            ext = filename.rsplit('.', 1)[-1].lower()
            if ext in ['xyz', 'cube', 'json', 'cif']:
                return ext

        if 'POSCAR' in basename or 'CONTCAR' in basename:
            return 'vasp'
        if 'OUTCAR' in basename:
            return 'vasp-out'
        if 'XDATCAR' in basename:
            return 'vasp-xdatcar'
        if 'vasp' in basename and basename.endswith('.xml'):
            return 'vasp-xml'
        if basename == 'coord':
            return 'turbomole'
        if basename == 'gradient':
            return 'turbomole-gradient'
        if basename.endswith('I_info'):
            return 'cmdft'
        if basename == 'atoms.dat':
            return 'iwm'

        if not read:
            return extension2format.get(ext, ext)

        fd = open(filename, 'rb')
    else:
        fd = filename
        if fd is sys.stdin:
            return 'json'

    data = fd.read(50000)
    if fd is not filename:
        fd.close()
    else:
        fd.seek(0)

    if len(data) == 0:
        raise IOError('Empty file: ' + filename)

    for format, magic in [('traj', b'- of UlmASE-Trajectory'),
                          ('traj', b'AFFormatASE-Trajectory'),
                          ('gpw', b'- of UlmGPAW'),
                          ('gpw', b'AFFormatGPAW'),
                          ('trj', b'PickleTrajectory'),
                          ('etsf', b'CDF'),
                          ('turbomole', b'$coord'),
                          ('turbomole-gradient', b'$grad'),
                          ('dftb', b'Geometry')]:
        if data.startswith(magic):
            return format

    for format, magic in [('gpaw-out', b'  ___ ___ ___ _ _ _'),
                          ('espresso-in', b'\n&system'),
                          ('espresso-in', b'\n&SYSTEM'),
                          ('aims-output', b'Invoking FHI-aims ...'),
                          ('lammps-dump', b'\nITEM: TIMESTEP\n'),
                          ('xsf', b'\nANIMSTEPS'),
                          ('xsf', b'\nCRYSTAL'),
                          ('xsf', b'\nSLAB'),
                          ('xsf', b'\nPOLYMER'),
                          ('xsf', b'\nMOLECULE'),
                          ('xsf', b'\nATOMS'),
                          ('dacapo-text',
                           b'&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&\n')]:
        if magic in data:
            return format

    return extension2format.get(ext, ext)


if __name__ == '__main__':
    import optparse
    parser = optparse.OptionParser(
        usage='python -m ase.io.formats file ...',
        description='Determine file type(s).')
    opts, filenames = parser.parse_args()
    if filenames:
        n = max(len(filename) for filename in filenames) + 2
    for filename in filenames:
        format = filetype(filename)
        if format and format in all_formats:
            description, code = all_formats[format]
        else:
            format = '?'
            description = '?'

        print('{0:{1}}{2} ({3})'.format(filename + ':', n,
                                        description, format))



from __future__ import division
from math import cos, sin, sqrt
from os.path import basename

import numpy as np

from ase.data import chemical_symbols
from ase.data.colors import jmol_colors
from ase.geometry import complete_cell
from ase.gui.repeat import Repeat
from ase.gui.rotate import Rotate
from ase.gui.render import Render
from ase.gui.colors import ColorWindow
from ase.utils import rotate


GREEN = '#DDFFDD'


class View:
    def __init__(self, rotations):
        self.colormode = 'jmol'  # The default colors
        self.nselected = 0
        self.labels = None
        self.axes = rotate(rotations)
        self.configured = False
        self.frame = None

    def set_coordinates(self, frame=None, focus=None):
        if frame is None:
            frame = self.frame
        self.make_box()
        self.bind(frame)
        n = self.images.natoms
        self.X = np.empty((n + len(self.B1) + len(self.bonds), 3))
        self.set_frame(frame, focus=focus, init=True)

    def set_frame(self, frame=None, focus=False, init=False):
        if frame is None:
            frame = self.frame

        n = self.images.natoms

        if self.frame is not None and self.frame > self.images.nimages:
            self.frame = self.images.nimages - 1

        if init or frame != self.frame:
            A = self.images.A
            nc = len(self.B1)
            nb = len(self.bonds)

            if init or (A[frame] != A[self.frame]).any():
                self.X[n:n + nc] = np.dot(self.B1, A[frame])
                self.B = np.empty((nc + nb, 3))
                self.B[:nc] = np.dot(self.B2, A[frame])

            if nb > 0:
                P = self.images.P[frame]
                Af = self.images.repeat[:, np.newaxis] * A[frame]
                a = P[self.bonds[:, 0]]
                b = P[self.bonds[:, 1]] + np.dot(self.bonds[:, 2:], Af) - a
                d = (b**2).sum(1)**0.5
                r = 0.65 * self.images.r
                x0 = (r[self.bonds[:, 0]] / d).reshape((-1, 1))
                x1 = (r[self.bonds[:, 1]] / d).reshape((-1, 1))
                self.X[n + nc:] = a + b * x0
                b *= 1.0 - x0 - x1
                b[self.bonds[:, 2:].any(1)] *= 0.5
                self.B[nc:] = self.X[n + nc:] + b

            filenames = self.images.filenames
            filename = filenames[frame]
            if (self.frame is None or
                filename != filenames[self.frame] or
                filename is None):
                if filename is None:
                    filename = 'ase.gui'
            filename = basename(filename)
            self.window.title = filename

        self.frame = frame
        self.X[:n] = self.images.P[frame]
        self.R = self.X[:n]
        if focus:
            self.focus()
        else:
            self.draw()

    def set_colors(self):
        self.colormode = 'jmol'
        self.colors = {}
        for z in np.unique(self.images.Z):
            rgb = jmol_colors[z]
            self.colors[z] = ('#{0:02X}{1:02X}{2:02X}'
                              .format(*(int(x * 255) for x in rgb)))

    def make_box(self):
        if not self.window['toggle-show-unit-cell']:
            self.B1 = self.B2 = np.zeros((0, 3))
            return

        V = self.images.A[0]
        nn = []
        for c in range(3):
            v = V[c]
            d = sqrt(np.dot(v, v))
            if d < 1e-12:
                n = 0
            else:
                n = max(2, int(d / 0.3))
            nn.append(n)
        self.B1 = np.zeros((2, 2, sum(nn), 3))
        self.B2 = np.zeros((2, 2, sum(nn), 3))
        n1 = 0
        for c, n in enumerate(nn):
            n2 = n1 + n
            h = 1.0 / (2 * n - 1)
            R = np.arange(n) * (2 * h)

            for i, j in [(0, 0), (0, 1), (1, 0), (1, 1)]:
                self.B1[i, j, n1:n2, c] = R
                self.B1[i, j, n1:n2, (c + 1) % 3] = i
                self.B1[i, j, n1:n2, (c + 2) % 3] = j
            self.B2[:, :, n1:n2] = self.B1[:, :, n1:n2]
            self.B2[:, :, n1:n2, c] += h
            n1 = n2
        self.B1.shape = (-1, 3)
        self.B2.shape = (-1, 3)

    def bind(self, frame):
        if not self.window['toggle-show-bonds']:
            self.bonds = np.empty((0, 5), int)
            return

        from ase.atoms import Atoms
        from ase.neighborlist import NeighborList
        nl = NeighborList(self.images.r * 1.5, skin=0, self_interaction=False)
        nl.update(Atoms(positions=self.images.P[frame],
                        cell=(self.images.repeat[:, np.newaxis] *
                              self.images.A[frame]),
                        pbc=self.images.pbc))
        nb = nl.nneighbors + nl.npbcneighbors

        bonds = np.empty((nb, 5), int)
        self.coordination = np.zeros((self.images.natoms), dtype=int)
        if nb == 0:
            return

        n1 = 0
        for a in range(self.images.natoms):
            indices, offsets = nl.get_neighbors(a)
            self.coordination[a] += len(indices)
            for a2 in indices:
                self.coordination[a2] += 1
            n2 = n1 + len(indices)
            bonds[n1:n2, 0] = a
            bonds[n1:n2, 1] = indices
            bonds[n1:n2, 2:] = offsets
            n1 = n2

        i = bonds[:n2, 2:].any(1)
        pbcbonds = bonds[:n2][i]
        bonds[n2:, 0] = pbcbonds[:, 1]
        bonds[n2:, 1] = pbcbonds[:, 0]
        bonds[n2:, 2:] = -pbcbonds[:, 2:]
        self.bonds = bonds

    def toggle_show_unit_cell(self, key=None):
        self.set_coordinates()

    def show_labels(self):
        index = self.window['show-labels']
        if index == 0:
            self.labels = None
        elif index == 1:
            self.labels = ([list(range(self.images.natoms))] *
                           self.images.nimages)
        elif index == 2:
            self.labels = self.images.M
        else:
            self.labels = [[chemical_symbols[x]
                            for x in self.images.Z]] * self.images.nimages

        self.draw()

    def toggle_show_axes(self, key=None):
        self.draw()

    def toggle_show_bonds(self, key=None):
        self.set_coordinates()

    def toggle_show_velocities(self, key=None):
        # XXX hard coded scale is ugly
        self.show_vectors(10 * np.nan_to_num(self.images.V))
        self.draw()

    def toggle_show_forces(self, key=None):
        self.show_vectors(np.nan_to_num(self.images.F))
        self.draw()

    def hide_selected(self):
        self.images.visible[self.images.selected] = False
        self.draw()

    def show_selected(self):
        self.images.visible[self.images.selected] = True
        self.draw()

    def repeat_window(self, key=None):
        Repeat(self)

    def rotate_window(self):
        return Rotate(self)

    def colors_window(self, key=None):
        win = ColorWindow(self)
        self.register_vulnerable(win)
        return win

    def focus(self, x=None):
        cell = (self.window['toggle-show-unit-cell'] and
                self.images.A[0].any())
        if (self.images.natoms == 0 and not cell):
            self.scale = 1.0
            self.center = np.zeros(3)
            self.draw()
            return

        P = np.dot(self.X, self.axes)
        n = self.images.natoms
        P[:n] -= self.images.r[:, None]
        P1 = P.min(0)
        P[:n] += 2 * self.images.r[:, None]
        P2 = P.max(0)
        self.center = np.dot(self.axes, (P1 + P2) / 2)
        S = 1.3 * (P2 - P1)
        w, h = self.window.size
        if S[0] * h < S[1] * w:
            self.scale = h / S[1]
        elif S[0] > 0.0001:
            self.scale = w / S[0]
        else:
            self.scale = 1.0
        self.draw()

    def reset_view(self, menuitem):
        self.axes = rotate('0.0x,0.0y,0.0z')
        self.set_coordinates()
        self.focus(self)

    def set_view(self, key):
        if key == 'Z':
            self.axes = rotate('0.0x,0.0y,0.0z')
        elif key == 'X':
            self.axes = rotate('-90.0x,-90.0y,0.0z')
        elif key == 'Y':
            self.axes = rotate('90.0x,0.0y,90.0z')
        elif key == 'Alt+Z':
            self.axes = rotate('180.0x,0.0y,90.0z')
        elif key == 'Alt+X':
            self.axes = rotate('0.0x,90.0y,0.0z')
        elif key == 'Alt+Y':
            self.axes = rotate('-90.0x,0.0y,0.0z')
        else:
            if key == '3':
                i, j = 0, 1
            elif key == '1':
                i, j = 1, 2
            elif key == '2':
                i, j = 2, 0
            elif key == 'Alt+3':
                i, j = 1, 0
            elif key == 'Alt+1':
                i, j = 2, 1
            elif key == 'Alt+2':
                i, j = 0, 2

            A = complete_cell(self.images.A[self.frame])
            x1 = A[i]
            x2 = A[j]

            norm = np.linalg.norm

            x1 = x1 / norm(x1)
            x2 = x2 - x1 * np.dot(x1, x2)
            x2 /= norm(x2)
            x3 = np.cross(x1, x2)

            self.axes = np.array([x1, x2, x3]).T

        self.set_coordinates()

    def get_colors(self, rgb=False):
        if rgb:
            return [tuple(int('0x' + rgb[i:i + 2])
                          for i in range(1, 7, 2))
                    for rgb in self.get_colors()]

        if self.colormode == 'jmol':
            return [self.colors[Z] for Z in self.images.Z]

        scalars = self.get_color_scalars()
        colorscale, cmin, cmax = self.colormode_data
        N = len(colorscale)
        indices = np.clip(((scalars - cmin) / (cmax - cmin) * N +
                           0.5).astype(int),
                          0, N - 1)
        return [colorscale[i] for i in indices]

    def get_color_scalars(self, frame=None):
        i = frame or self.frame

        if self.colormode == 'tag':
            return self.images.T[i]
        if self.colormode == 'force':
            f = (self.images.F[i]**2).sum(1)**0.5
            return f * self.images.dynamic
        elif self.colormode == 'velocity':
            return (self.images.V[i]**2).sum(1)**0.5
        elif self.colormode == 'charge':
            return self.images.q[i]
        elif self.colormode == 'magmom':
            return self.images.M[i]

    def draw(self, status=True):
        self.window.clear()
        axes = self.scale * self.axes * (1, -1, 1)
        offset = np.dot(self.center, axes)
        offset[:2] -= 0.5 * self.window.size
        X = np.dot(self.X, axes) - offset
        n = self.images.natoms
        self.indices = X[:, 2].argsort()
        if self.window['toggle-show-bonds']:
            r = self.images.r * (0.65 * self.scale)
        else:
            r = self.images.r * self.scale
        P = self.P = X[:n, :2]
        A = (P - r[:, None]).round().astype(int)
        X1 = X[n:, :2].round().astype(int)
        X2 = (np.dot(self.B, axes) - offset).round().astype(int)
        disp = (np.dot(self.images.D[self.frame], axes)).round().astype(int)
        d = (2 * r).round().astype(int)

        vectors = (self.window['toggle-show-velocities'] or
                   self.window['toggle-show-forces'])
        if vectors:
            V = np.dot(self.vectors[self.frame], axes) + X[:n]

        colors = self.get_colors()
        circle = self.window.circle
        line = self.window.line
        dynamic = self.images.dynamic
        selected = self.images.selected
        visible = self.images.visible
        ncell = len(self.B1)
        bw = self.scale * 0.15
        for a in self.indices:
            if a < n:
                ra = d[a]
                if visible[a]:
                    # Draw the atoms
                    if self.moving and selected[a]:
                        circle(GREEN, False,
                               A[a, 0] - 4, A[a, 1] - 4,
                               A[a, 0] + ra + 4, A[a, 1] + ra + 4)

                    circle(colors[a], selected[a],
                           A[a, 0], A[a, 1], A[a, 0] + ra, A[a, 1] + ra)

                    # Draw labels on the atoms
                    if self.labels is not None:
                        self.window.text(A[a, 0], A[a, 1],
                                         str(self.labels[self.frame][a]))

                    # Draw cross on constrained atoms
                    if not dynamic[a]:
                        R1 = int(0.14644 * ra)
                        R2 = int(0.85355 * ra)
                        line((A[a, 0] + R1, A[a, 1] + R1,
                              A[a, 0] + R2, A[a, 1] + R2))
                        line((A[a, 0] + R2, A[a, 1] + R1,
                              A[a, 0] + R1, A[a, 1] + R2))

                    # Draw velocities or forces
                    if vectors:
                        line((X[a, 0], X[a, 1], V[a, 0], V[a, 1]), width=2)
            else:
                # Draw unit cell and/or bonds:
                a -= n
                if a < ncell:
                    line((X1[a, 0] + disp[0], X1[a, 1] + disp[1],
                          X2[a, 0] + disp[0], X2[a, 1] + disp[1]))
                else:
                    line((X1[a, 0] + disp[0], X1[a, 1] + disp[1],
                          X2[a, 0] + disp[0], X2[a, 1] + disp[1]), width=bw)

        if self.window['toggle-show-axes']:
            self.draw_axes()

        if self.images.nimages > 1:
            self.draw_frame_number()

        self.window.update()

        if status:
            self.status()

    def draw_axes(self):
        axes_length = 15

        rgb = ['red', 'green', 'blue']

        for i in self.axes[:, 2].argsort():
            a = 20
            b = self.window.size[1] - 20
            c = int(self.axes[i][0] * axes_length + a)
            d = int(-self.axes[i][1] * axes_length + b)
            self.window.line((a, b, c, d))
            self.window.text(c, d, 'XYZ'[i], color=rgb[i])

    def draw_frame_number(self):
        x, y = self.window.size
        self.window.text(x, y, '{0}/{1}'.format(self.frame,
                                                self.images.nimages),
                         anchor='SE')

    def release(self, event):
        if event.button in [4, 5]:
            self.scroll_event(event)
            return

        if event.button != 1:
            return

        selected = self.images.selected
        selected_ordered = self.images.selected_ordered

        if event.time < self.t0 + 200:  # 200 ms
            d = self.P - self.xy
            hit = np.less((d**2).sum(1), (self.scale * self.images.r)**2)
            for a in self.indices[::-1]:
                if a < self.images.natoms and hit[a]:
                    if event.modifier == 'ctrl':
                        selected[a] = not selected[a]
                        if selected[a]:
                            selected_ordered += [a]
                        elif len(selected_ordered) > 0:
                            if selected_ordered[-1] == a:
                                selected_ordered = selected_ordered[:-1]
                            else:
                                selected_ordered = []
                    else:
                        selected[:] = False
                        selected[a] = True
                        selected_ordered = [a]
                    break
            else:
                selected[:] = False
                selected_ordered = []
            self.draw()
        else:
            A = (event.x, event.y)
            C1 = np.minimum(A, self.xy)
            C2 = np.maximum(A, self.xy)
            hit = np.logical_and(self.P > C1, self.P < C2)
            indices = np.compress(hit.prod(1), np.arange(len(hit)))
            if event.modifier != 'ctrl':
                selected[:] = False
            selected[indices] = True
            if (len(indices) == 1 and
                indices[0] not in self.images.selected_ordered):
                selected_ordered += [indices[0]]
            elif len(indices) > 1:
                selected_ordered = []
            self.draw()

        indices = np.arange(self.images.natoms)[self.images.selected]
        if len(indices) != len(selected_ordered):
            selected_ordered = []
        self.images.selected_ordered = selected_ordered

    def press(self, event):
        self.button = event.button
        self.xy = (event.x, event.y)
        self.t0 = event.time
        self.axes0 = self.axes
        self.center0 = self.center

    def move(self, event):
        x = event.x
        y = event.y
        x0, y0 = self.xy
        if self.button == 1:
            x0 = int(round(x0))
            y0 = int(round(y0))
            self.draw()
            self.window.canvas.create_rectangle((x, y, x0, y0))
            return

        if event.modifier == 'shift':
            self.center = (self.center0 -
                           np.dot(self.axes, (x - x0, y0 - y, 0)) / self.scale)
        else:
            # Snap mode: the a-b angle and t should multipla of 15 degrees ???
            a = x - x0
            b = y0 - y
            t = sqrt(a * a + b * b)
            if t > 0:
                a /= t
                b /= t
            else:
                a = 1.0
                b = 0.0
            c = cos(0.01 * t)
            s = -sin(0.01 * t)
            rotation = np.array([(c * a * a + b * b, (c - 1) * b * a, s * a),
                                 ((c - 1) * a * b, c * b * b + a * a, s * b),
                                 (-s * a, -s * b, c)])
            self.axes = np.dot(self.axes0, rotation)
            if self.images.natoms > 0:
                com = self.X[:self.images.natoms].mean(0)
            else:
                com = self.images.A[self.frame].mean(0)
            self.center = com - np.dot(com - self.center0,
                                       np.dot(self.axes0, self.axes.T))
        self.draw(status=False)

    def render_window(self, action):
        Render(self)

    def show_vectors(self, vectors):
        self.vectors = vectors

    def resize(self, event):
        w, h = self.window.size
        self.scale *= (event.width * event.height / (w * h))**0.5
        self.window.size[:] = [event.width, event.height]
        self.draw()



from __future__ import unicode_literals
import argparse
import os

import numpy as np

from ase import Atoms
from ase.calculators.singlepoint import SinglePointCalculator
from ase.build import molecule
from ase.gui.i18n import _
from ase.test import NotAvailable

try:
    import ase.gui.ui as ui
except ImportError:
    raise NotAvailable

from ase.gui.gui import GUI


if not os.environ.get('DISPLAY'):
    raise NotAvailable


class Error:
    """Fake window for testing puposes."""
    has_been_called = False

    def __call__(self, title, text=None):
        self.text = text or title
        self.has_been_called = True

    def called(self, text=None):
        """Check that an oops-window was opened with correct title."""
        if not self.has_been_called:
            return False

        self.has_been_called = False  # ready for next call

        return text is None or text == self.text


ui.error = Error()

alltests = []


def test(f):
    """Decorator for marking tests."""
    alltests.append(f.__name__)
    return f


@test
def nanotube(gui):
    nt = gui.nanotube_window()
    nt.apply()
    nt.element[1].value = '?'
    nt.apply()
    assert ui.error.called(
        _('You have not (yet) specified a consistent set of parameters.'))

    nt.element[1].value = 'C'
    nt.ok()
    assert gui.images.natoms == 20


@test
def nanopartickle(gui):
    n = gui.nanoparticle_window()
    n.element.symbol = 'Cu'
    n.apply()
    n.set_structure_data()
    assert gui.images.natoms == 675
    n.method.value = 'wulff'
    n.update_gui_method()
    n.apply()


@test
def color(gui):
    a = Atoms('C10', magmoms=np.linspace(1, -1, 10))
    a.positions[:] = np.linspace(0, 9, 10)[:, None]
    a.calc = SinglePointCalculator(a, forces=a.positions)
    gui.new_atoms(a)
    c = gui.colors_window()
    c.toggle('force')
    text = c.toggle('magmom')
    assert [button.active for button in c.radio.buttons] == [1, 0, 1, 0, 0, 1]
    assert text.rsplit('[', 1)[1].startswith('-1.000000,1.000000]')


@test
def settings(gui):
    gui.new_atoms(molecule('H2O'))
    s = gui.settings()
    s.scale.value = 1.9
    s.scale_radii()


@test
def rotate(gui):
    gui.new_atoms(molecule('H2O'))
    gui.rotate_window()


# @test not ready yet!
def open(gui):
    molecule('H2O').write('h2o.json')
    gui.open()


p = argparse.ArgumentParser()
p.add_argument('tests', nargs='*')
p.add_argument('-p', '--pause', action='store_true')

if __name__ == '__main__':
    args = p.parse_args()
else:
    # We are running inside the test framework: ignore sys.args
    args = p.parse_args([])

for name in args.tests or alltests:
    for n in alltests:
        if n.startswith(name):
            name = n
            break
    else:
        1 / 0
    print(name)
    test = globals()[name]
    gui = GUI()

    def f():
        test(gui)
        if not args.pause:
            gui.exit()
    gui.run(test=f)



"""
  Module containing the routine to read the mbpt_lcao output
"""


from __future__ import division
import ase.io as aio
import numpy as np


"""
  class read_mbpt_lcao_output: Main class of the library
          use to load the data from the TDDFT code
  class MBPT_LCAO_Parameters: class that fix the input parameter
          concerning loading data
  class MBPT_LCAO_Properties_figure: class that fix the properties
          concerning the figures
"""


class read_mbpt_lcao_output:
    """
    Top class, to be call by a script.
    Read the output data from the mbpt_lcao program

    Parameters
    ----------
        No input paramters, but the args, and prop variable have
        to be modify as function of your wishes

    References
    ----------

    Example
    -------
    """

    def __init__(self):

        self.args = MBPT_LCAO_Parameters()
        self.prop = MBPT_LCAO_Properties_figure()

    def Read(self, YFname=None):
        if self.args.folder != './' and \
                self.prop.fatoms == 'domiprod_atom2coord.xyz':
            self.prop.fatoms = self.args.folder + self.prop.fatoms
        self.args.check_input()
        if self.args.format_input == 'txt':
            output = read_text_data(self.args, self.prop, YFname)
        elif self.args.format_input == 'hdf5':
            output = read_hdf5_data(self.args, self.prop, YFname)
        else:
            raise ValueError('no other format supported')
        return output


class read_hdf5_data:
    """
        read data from mbpt_lcao calculation saved in the hdf5 format
    """

    def __init__(self, args, prop_fig, YFname):

        try:
            import h5py
        except:
            raise ValueError('The module need h5py library in order \
                        to read hdf5 files!')

        self.atoms = aio.read(prop_fig.fatoms)
        self.determine_fname(args, YFname)

        self.File = h5py.File(self.fname, 'r')

        if args.tem_iter == 'iter':
            self.check_file_iter(args)
            self.extract_data_iter(args)
        elif args.tem_iter == 'tem':
            self.check_file_tem(args)
            self.extract_data_tem(args)
        else:
            raise ValueError('only tem or iter!!')

        self.set_carac(args, prop_fig)

    def set_carac(self, args_p, prop):
        """
            set the ylabel for the plot and other parameters
        """
        if args_p.quantity == 'intensity':
            self.ylabel = r'$|\frac{E}{E_{0}}|^{2}$'
        elif args_p.quantity == 'density':
            if args_p.ReIm == 'im':
                self.ylabel = r'$Im(\delta n)$'
            elif args_p.ReIm == 're':
                self.ylabel = r'$Re(\delta n)$'
        else:
            self.ylabel = 'Intensity (a.u.)'

        self.pl_num = str(prop.plan_coord)
        self.pl_file = prop.plan_coord
        for i in range(len(self.pl_num)):
            if self.pl_num[i] == '.':
                self.pl_file = self.pl_num[0:i] + '-' + \
                    self.pl_num[i + 1:len(self.pl_num)]

    def determine_fname(self, args_p, perso=None):
        """
        determine the files name
        """

        if perso is None:
            self.fname = args_p.folder + 'tddft_' + \
                args_p.tem_iter + '_output.hdf5'
        else:
            self.fname = args_p.folder + perso

        print(self.fname)

    def check_file_tem(self, args_p):
        """
        Check if the file tddft_tem_output.hdf5 caontians the wished data
        """

        field_spatial = ['dens', 'potential', 'intensity', 'efield']
        if args_p.quantity == 'spectrum':
            if args_p.tem_input['tem_method'] == 'C':
                quantity = 'tem_spectrum_cc'
            else:
                quantity = 'tem_spectrum'
        elif args_p.quantity == 'E_loss':
            self.dname = args_p.quantity + '_' + args_p.inter
            quantity = None
        else:
            if args_p.time == 0:
                print(args_p.tem_input['tem_method'])
                if args_p.tem_input['tem_method'] == 'C':
                    quantity = 'field_spatial_cc_freq_{0:.2f}'\
                        .format(args_p.plot_freq)
                else:
                    quantity = 'field_spatial_freq_{0:.2f}'\
                        .format(args_p.plot_freq)
            else:
                quantity = 'field_time'

        print('quantity: ', quantity)
        chain = '_v{0:.8f}_bx{1:.8f}_by{2:.8f}_bz{3:.8f}'\
            .format(args_p.tem_input['vnorm'], args_p.tem_input['b'][0],
                    args_p.tem_input['b'][1], args_p.tem_input['b'][2])
        if quantity is None:
            self.group = self.File['/']
        else:
            self.group = self.File[quantity]
            if args_p.quantity == 'spectrum':
                self.dname = 'tem_spectrum_' + args_p.inter + chain
            elif args_p.quantity in field_spatial or \
                    quantity == 'field_time':
                self.group = self.group['tem_' + args_p.inter + chain]

                if args_p.quantity in field_spatial:
                    if args_p.quantity == 'intensity':
                        self.dname = args_p.quantity
                    else:
                        self.dname = args_p.quantity + '_' + args_p.ReIm
                else:
                    self.dname = []
                    if args_p.quantity == 'intensity':
                        for i in range(args_p.time_num):
                            self.dname.append('data_time_{0}/'
                                              .format(i) + args_p.quantity)
                    else:
                        for i in range(args_p.time_num):
                            self.dname.append(
                                'data_time_{0}/' .format(i) +
                                args_p.quantity + '_' + args_p.ReIm)

        if isinstance(self.dname, list):
            for i, name in enumerate(self.dname):
                print(name)
                sub_group = self.group['data_time_{0}/'.format(i)]
                if args_p.quantity not in sub_group.keys():
                    raise ValueError(
                        name +
                        ' not saved in ' +
                        self.fname +
                        '. check with h5ls command.')
        else:
            if self.dname not in self.group.keys():
                print(self.group.keys())
                raise ValueError(self.dname + ' not saved in ' +
                                 self.fname + '. check with h5ls command.')

    def extract_data_tem(self, args_p):
        """
        Extract the data into the foloowing structures,
        Array: contains the data that you wish to get
        freq: frequency range if polarizability

        geometrical data
        dr: the spatial step
        origin: origin of the system
        lbound: lower bound of the array
        ubound: upper bound of the array
        dim: shape of the array
        mesh: the 2D or 3D mesh for the plotting
        """
        if args_p.quantity == 'spectrum':
            self.Array = self.group[self.dname].value[1, :]
            self.freq = self.group[self.dname].value[0, :]
        elif args_p.quantity == 'E_loss':
            self.Array = self.group[self.dname].value
        else:
            keys_f = ['dr', 'origin', 'ibox']
            keys = ['dr', 'origin', 'ibox']
            dico = {}

            for i, k in enumerate(keys):
                dico[k] = self.group[keys_f[i]].value

            self.dr = dico['dr']
            self.origin = dico['origin']
            self.lbound = dico['ibox'][:, 0]
            self.ubound = dico['ibox'][:, 1]

            if args_p.time == 0:
                keys_f.append(self.dname)

                dico['Array'] = self.group[keys_f[3]].value

                dim = dico['Array'].shape[::-1]
                if args_p.quantity == 'Efield':
                    self.Array = dico['Array'].ravel('F').reshape(
                        dim[0], dim[1], dim[2], dim[3])
                else:
                    self.Array = dico['Array'].T

            else:
                sh = self.group[self.dname[0]].value.shape
                self.t = self.group['t'].value
                if args_p.quantity == 'Efield':
                    self.Array = np.array(
                        (self.t.shape[0], sh[0], sh[1], sh[2], sh[3]),
                        dtype=float)
                    for i in range(len(self.dname)):
                        self.Array[i] = self.group[self.dname[i]].value
                else:
                    self.Array = np.zeros(
                        (self.t.shape[0], sh[0], sh[1], sh[2]), dtype=float)
                    for i in range(len(self.dname)):
                        print('array ', self.Array.shape)
                        print('sum(data) ',
                              np.sum(abs(self.group[self.dname[i]].value)))
                        self.Array[i] = self.group[self.dname[i]].value

            self.determine_box()
            self.mesh3D()
            self.mesh2D()

            self.xy_prof = self.xy_mesh[:, int(self.xy_mesh.shape[1] / 2)]
            self.xz_prof = self.xz_mesh[int(self.xz_mesh.shape[0] / 2), :]

            self.yx_prof = self.yx_mesh[:, int(self.yx_mesh.shape[1] / 2)]
            self.yz_prof = self.yz_mesh[int(self.yz_mesh.shape[0] / 2), :]

            self.zx_prof = self.zx_mesh[:, int(self.zx_mesh.shape[1] / 2)]
            self.zy_prof = self.zy_mesh[int(self.zy_mesh.shape[0] / 2), :]

    def check_file_iter(self, args_p):
        if args_p.quantity == 'polarizability':
            quantity = args_p.quantity
        else:
            quantity = 'field_spatial_freq_{0:.2f}_'.format(
                args_p.plot_freq) + args_p.inter

        if quantity not in self.File.keys():
            raise ValueError(
                quantity +
                ' not saved in ' +
                self.fname +
                '. check with h5ls command.')

        self.group = self.File['/' + quantity]

        if args_p.quantity == 'polarizability':
            self.dname = 'dipol_' + args_p.inter + '_iter_krylov'
        else:
            self.dname = args_p.quantity

        if self.dname == 'intensity':
            if self.dname not in self.group.keys():
                raise ValueError(
                    self.dname +
                    ' not saved in ' +
                    self.fname +
                    '. check with h5ls command.')
        elif self.dname == 'polarizability':
            if args_p.species != '':
                if self.dname + '_' + args_p.ReIm + '_' + \
                        args_p.species not in self.group.keys():
                    raise ValueError(
                        self.dname +
                        '_' +
                        args_p.ReIm +
                        '_' +
                        args_p.species +
                        ' not saved in ' +
                        self.fname +
                        '. check with h5ls command.')
            else:
                if self.dname + '_' + args_p.ReIm not in self.group.keys():
                    raise ValueError(
                        self.dname +
                        '_' +
                        args_p.ReIm +
                        ' not saved in ' +
                        self.fname +
                        '. check with h5ls command.')

        else:
            if self.dname + '_' + args_p.ReIm not in self.group.keys():
                raise ValueError(
                    self.dname +
                    '_' +
                    args_p.ReIm +
                    ' not saved in ' +
                    self.fname +
                    '. check with h5ls command.')

    def extract_data_iter(self, args_p):
        if args_p.quantity == 'polarizability':
            self.freq = self.group['frequency'].value
            if args_p.species != '':
                self.Array = self.group[
                    self.dname +
                    '_' +
                    args_p.ReIm +
                    '_' +
                    args_p.species].value.T
            else:
                self.Array = self.group[self.dname + '_' + args_p.ReIm].value.T
                for i in range(self.Array.shape[0]):
                    self.Array[i, :, :] = self.Array[i, :, :].T
        else:
            if self.dname == 'dens':
                keys_f = [
                    'dr',
                    'origin_dens',
                    'ibox_dens',
                    self.dname +
                    '_' +
                    args_p.ReIm]
            elif self.dname == 'intensity':
                keys_f = ['dr', 'origin', 'ibox', self.dname]
            else:
                keys_f = [
                    'dr',
                    'origin',
                    'ibox',
                    self.dname +
                    '_' +
                    args_p.ReIm]

            keys = ['dr', 'origin', 'ibox', 'Array']
            dico = {}

            for i, k in enumerate(keys):
                dico[k] = self.group[keys_f[i]].value

            self.dr = dico['dr']
            self.origin = dico['origin']
            self.lbound = dico['ibox'][:, 0]
            self.ubound = dico['ibox'][:, 1]

            dim = dico['Array'].shape[::-1]
            if args_p.quantity == 'Efield':
                self.Array = dico['Array'].ravel('F').reshape(
                    dim[0], dim[1], dim[2], dim[3])
            else:
                self.Array = dico['Array'].T

            self.determine_box()
            self.mesh3D()
            self.mesh2D()

            self.xy_prof = self.xy_mesh[:, int(self.xy_mesh.shape[1] / 2)]
            self.xz_prof = self.xz_mesh[int(self.xz_mesh.shape[0] / 2), :]

            self.yx_prof = self.yx_mesh[:, int(self.yx_mesh.shape[1] / 2)]
            self.yz_prof = self.yz_mesh[int(self.yz_mesh.shape[0] / 2), :]

            self.zx_prof = self.zx_mesh[:, int(self.zx_mesh.shape[1] / 2)]
            self.zy_prof = self.zy_mesh[int(self.zy_mesh.shape[0] / 2), :]

    def determine_box(self):
        box = list()
        box.append(self.dr * self.lbound + self.origin)
        box.append(self.dr * self.ubound + self.origin)
        self.box = np.array(box)
        self.dim = self.ubound - self.lbound + 1

    def mesh2D(self):

        self.xy_mesh = np.zeros((self.dim[1], self.dim[2]), dtype=float)
        self.xz_mesh = np.zeros((self.dim[1], self.dim[2]), dtype=float)
        for j in range(self.xy_mesh.shape[1]):
            for i in range(self.xy_mesh.shape[0]):
                self.xy_mesh[i, j] = self.box[0][1] + \
                    i * self.dr[1] + self.origin[1]

        for i in range(self.xz_mesh.shape[0]):
            for j in range(self.xz_mesh.shape[1]):
                self.xz_mesh[i, j] = self.box[0][2] + \
                    j * self.dr[2] + self.origin[2]

        self.yx_mesh = np.zeros((self.dim[0], self.dim[2]), dtype=float)
        self.yz_mesh = np.zeros((self.dim[0], self.dim[2]), dtype=float)
        for j in range(self.yx_mesh.shape[1]):
            for i in range(self.yx_mesh.shape[0]):
                self.yx_mesh[i, j] = self.box[0][0] + \
                    i * self.dr[0] + self.origin[0]

        for i in range(self.yz_mesh.shape[0]):
            for j in range(self.yz_mesh.shape[1]):
                self.yz_mesh[i, j] = self.box[0][2] + \
                    j * self.dr[2] + self.origin[2]

        self.zx_mesh = np.zeros((self.dim[0], self.dim[1]), dtype=float)
        self.zy_mesh = np.zeros((self.dim[0], self.dim[1]), dtype=float)
        for j in range(self.zx_mesh.shape[1]):
            for i in range(self.zx_mesh.shape[0]):
                self.zx_mesh[i, j] = self.box[0][0] + \
                    i * self.dr[0] + self.origin[0]

        for i in range(self.zy_mesh.shape[0]):
            for j in range(self.zy_mesh.shape[1]):
                self.zy_mesh[i, j] = self.box[0][1] + \
                    j * self.dr[1] + self.origin[1]

    def mesh3D(self):
        self.xmesh = np.zeros(
            (self.dim[0], self.dim[1], self.dim[2]), dtype=float)
        self.ymesh = np.zeros(
            (self.dim[0], self.dim[1], self.dim[2]), dtype=float)
        self.zmesh = np.zeros(
            (self.dim[0], self.dim[1], self.dim[2]), dtype=float)

        for i in range(self.xmesh.shape[0]):
            nb = self.box[0][0] + i * self.dr[0] + self.origin[0]
            self.xmesh[i, :, :] = nb

        for i in range(self.xmesh.shape[1]):
            nb = self.box[0][1] + i * self.dr[1] + self.origin[1]
            self.ymesh[:, i, :] = nb

        for i in range(self.xmesh.shape[2]):
            nb = self.box[0][2] + i * self.dr[2] + self.origin[2]
            self.zmesh[:, :, i] = nb


class read_text_data:
    """
    Class that read the output data of the tddft program for the
    field enhancement.
    can read .dat files (text file), .npy files (binary files) or
    .hdf5 files (binary).
    it is loading all the different parameter in array to plotting purpose.
    Input parameters:
    -----------------
    args: (class parameter), input parameter
    prop_fig: (class Properties_figure): Properties of the figure
    kwargs (optionnal):
      new_data: (list of args class!), list of the other args if needed
      to get more than one data
    Output parameters:
    ------------------
    all the data about the box save in:
    self.dr
    self.box
    self.Array
    self.dim
    self.mesh.....
    Function of the class:
    ----------------------
    initialise
    determine_fname
    recover_data
    readhdf5
    read_npy
    read_txt
    determine_box
    mesh3D
    """

    def __init__(self, args, prop_fig, YFname):

        if args.tem_iter == 'tem':
            raise ValueError('text format only with iter')

        self.atoms = aio.read(prop_fig.fatoms)
        self.fname = self.determine_fname(args, perso=YFname)
        self.dr, self.origin, self.lbound, self.ubound, self.Array, \
            self.box, self.dim = self.read_txt(args, self.fname)

        if args.quantity != 'polarizability':

            self.mesh3D()
            self.mesh2D()

            self.xy_prof = self.xy_mesh[:, int(self.xy_mesh.shape[1] / 2)]
            self.xz_prof = self.xz_mesh[int(self.xz_mesh.shape[0] / 2), :]

            self.yx_prof = self.yx_mesh[:, int(self.yx_mesh.shape[1] / 2)]
            self.yz_prof = self.yz_mesh[int(self.yz_mesh.shape[0] / 2), :]

            self.zx_prof = self.zx_mesh[:, int(self.zx_mesh.shape[1] / 2)]
            self.zy_prof = self.zy_mesh[int(self.zy_mesh.shape[0] / 2), :]

        self.set_carac(args, prop_fig)

    def set_carac(self, args_p, prop):
        if args_p.quantity == 'intensity':
            self.ylabel = r'$|\frac{E}{E_{0}}|^{2}$'
        elif args_p.quantity == 'density':
            if args_p.ReIm == 'im':
                self.ylabel = r'$Im(\delta n)$'
            elif args_p.ReIm == 're':
                self.ylabel = r'$Re(\delta n)$'
        else:
            self.ylabel = 'Intensity (a.u.)'

        self.pl_num = str(prop.plan_coord)
        self.pl_file = prop.plan_coord
        for i in range(len(self.pl_num)):
            if self.pl_num[i] == '.':
                self.pl_file = self.pl_num[0:i] + '-' + \
                    self.pl_num[i + 1:len(self.pl_num)]

    def determine_fname(self, args_p, perso=None):
        """
        set the files name
        """
        if perso is None:
            if args_p.quantity == 'Efield':
                fname = [
                    args_p.folder +
                    'e_field_' +
                    args_p.ReIm +
                    '.x_' +
                    args_p.inter +
                    args_p.tem_iter +
                    '.dat',
                    args_p.folder +
                    'e_field_' +
                    args_p.ReIm +
                    '.y_' +
                    args_p.inter +
                    args_p.tem_iter +
                    '.dat',
                    args_p.folder +
                    'e_field_' +
                    args_p.ReIm +
                    '.z_' +
                    args_p.inter +
                    args_p.tem_iter +
                    '.dat']
            elif args_p.quantity == 'intensity':
                fname = args_p.folder + args_p.quantity + '_' + \
                    args_p.inter + '_' + args_p.tem_iter + '.dat'
            elif args_p.quantity == 'polarizability':
                fname = args_p.folder + 'dipol_' + args_p.inter + '_' + \
                    args_p.tem_iter + '_krylov_' + args_p.ReIm + '.txt'
            else:
                fname = args_p.folder + args_p.quantity + '_' + args_p.ReIm + \
                    '_' + args_p.inter + '_' + args_p.tem_iter + '.dat'
        else:
            fname = args_p.folder + perso

        print(fname)
        return fname

    def read_txt(self, args_p, fname):
        from mbpt_lcao_utils import read_file, str2float
        data = list()
        dim = list()
        end_box = 10

        if args_p.quantity == 'Efield':
            A = []
            for i in enumerate(fname):
                LINE = read_file(i[1])

                for j in range(end_box):
                    if LINE[j][1] != '#':
                        nb = str2float(LINE[j])
                        data.append(nb)

                dr = np.array(data[0])
                origin = np.array(data[1])
                lbound = np.array(data[2])
                ubound = np.array(data[3])
                box, dim = self.determine_box(dr, ubound, lbound, origin)

                A.append(np.zeros((dim[0], dim[1], dim[2]), dtype=float))

                l = end_box
                for k in range(int(dim[2])):
                    for j in range(int(dim[1])):
                        A[i[0]][:, j, k] = np.array(str2float(LINE[l]))
                        l = l + 1

            Array = np.zeros(
                (A[0].shape[0],
                 A[0].shape[1],
                    A[0].shape[2],
                    3),
                dtype=float)
            Array[:, :, :, 0] = A[0]
            Array[:, :, :, 1] = A[1]
            Array[:, :, :, 2] = A[2]

        elif args_p.quantity == 'polarizability':
            self.freq = np.loadtxt(fname)[:, 0]
            Array = np.loadtxt(fname)[:, 2:11].reshape(
                self.freq.shape[0], 3, 3)
            dr = 0.0
            origin = 0.0
            lbound = 0.0
            ubound = 0.0
            box = 0.0
            dim = 0.0
        else:
            LINE = read_file(fname)

            for i in range(end_box):
                if LINE[i][1] != '#':
                    nb = str2float(LINE[i])
                    data.append(nb)

            dr = np.array(data[0])
            origin = np.array(data[1])
            lbound = np.array(data[2])
            ubound = np.array(data[3])
            box, dim = self.determine_box(dr, ubound, lbound, origin)

            Array = np.zeros((dim[0], dim[1], dim[2]), dtype=float)

            l = end_box
            for k in range(int(dim[2])):
                for j in range(int(dim[1])):
                    Array[:, j, k] = np.array(str2float(LINE[l]))
                    l = l + 1

        return dr, origin, lbound, ubound, Array, box, dim

    def determine_box(self, dr, ubound, lbound, origin):
        box = list()
        box.append(dr * lbound + origin)
        box.append(dr * ubound + origin)

        dim = ubound - lbound + 1

        return box, dim

    def mesh2D(self):

        self.xy_mesh = np.zeros((self.dim[1], self.dim[2]), dtype=float)
        self.xz_mesh = np.zeros((self.dim[1], self.dim[2]), dtype=float)
        for j in range(self.xy_mesh.shape[1]):
            for i in range(self.xy_mesh.shape[0]):
                self.xy_mesh[i, j] = self.box[0][1] + \
                    i * self.dr[1] + self.origin[1]

        for i in range(self.xz_mesh.shape[0]):
            for j in range(self.xz_mesh.shape[1]):
                self.xz_mesh[i, j] = self.box[0][2] + \
                    j * self.dr[2] + self.origin[2]

        self.yx_mesh = np.zeros((self.dim[0], self.dim[2]), dtype=float)
        self.yz_mesh = np.zeros((self.dim[0], self.dim[2]), dtype=float)
        for j in range(self.yx_mesh.shape[1]):
            for i in range(self.yx_mesh.shape[0]):
                self.yx_mesh[i, j] = self.box[0][0] + \
                    i * self.dr[0] + self.origin[0]

        for i in range(self.yz_mesh.shape[0]):
            for j in range(self.yz_mesh.shape[1]):
                self.yz_mesh[i, j] = self.box[0][2] + \
                    j * self.dr[2] + self.origin[2]

        self.zx_mesh = np.zeros((self.dim[0], self.dim[1]), dtype=float)
        self.zy_mesh = np.zeros((self.dim[0], self.dim[1]), dtype=float)
        for j in range(self.zx_mesh.shape[1]):
            for i in range(self.zx_mesh.shape[0]):
                self.zx_mesh[i, j] = self.box[0][0] + \
                    i * self.dr[0] + self.origin[0]

        for i in range(self.zy_mesh.shape[0]):
            for j in range(self.zy_mesh.shape[1]):
                self.zy_mesh[i, j] = self.box[0][1] + \
                    j * self.dr[1] + self.origin[1]

    def mesh3D(self):
        self.xmesh = np.zeros(
            (self.dim[0], self.dim[1], self.dim[2]), dtype=float)
        self.ymesh = np.zeros(
            (self.dim[0], self.dim[1], self.dim[2]), dtype=float)
        self.zmesh = np.zeros(
            (self.dim[0], self.dim[1], self.dim[2]), dtype=float)

        for i in range(self.xmesh.shape[0]):
            nb = self.box[0][0] + i * self.dr[0] + self.origin[0]
            self.xmesh[i, :, :] = nb

        for i in range(self.xmesh.shape[1]):
            nb = self.box[0][1] + i * self.dr[1] + self.origin[1]
            self.ymesh[:, i, :] = nb

        for i in range(self.xmesh.shape[2]):
            nb = self.box[0][2] + i * self.dr[2] + self.origin[2]
            self.zmesh[:, :, i] = nb


class MBPT_LCAO_Parameters:
    """
    Contains the input parameters use by plot_3D in plot_lib.py
    The parameters are:
    self.quantity (string, default:'polarizability'): the type of data
    than one wish to plot, can be
                                      intensity
                                      efield
                                      potential
                                      dens
                                      polarizability
                                      spectrum
                                      E_loss
    self.interacting (int, default: 0) interacting or non-interacting data,
                                      depend of your TDDFT calculation
    self.ReIm (string, default: 'im'): plot imaginary or real part
    self.format_input (string, default:'.hdf5'): format of the input file,
    can be
                                                .dat
                                                .npy
                                                .hdf5
    self.folder (string, default: './'): name of the folder where are
                                         save the input data
    self.time (int, default: 1): time plotting, 0 or 1
    self.time_num (int, default: 0): time index than one which to plot
    self.tem_iter (int, default: 0) : tem or iter plotting, 0 => iter, 1 => tem
    self.movie (int, default: 0) : if tem and plot_dens_time=1
                                   (in tddft_lr.inp) then do a movie
    """

    def __init__(self):
        self.quantity = 'polarizability'
        self.inter = 'inter'
        self.ReIm = 'im'
        self.format_input = 'hdf5'
        self.folder = './'
        self.time = 0
        self.time_num = 0
        self.tem_iter = 'iter'
        self.species = ''
        self.movie = 0
        self.plot_freq = 0.0
        self.tem_input = {'dr': np.array([0.3,
                                          0.3,
                                          0.3]),
                          'vnorm': 1.0,
                          'v': np.array([1.0,
                                         0.0,
                                         0.0]),
                          'dw': 0.1,
                          'b': np.array([0.0,
                                         0.0,
                                         0.0]),
                          'vrange': None,
                          'brange': None,
                          'tem_method': 'N'}
        self.exportData = {
            'export': False,
            'dtype': 'HDF5',
            'fname': 'exportData'}
        self.kwargs = {}  # to add more arguments

    def check_input(self):
        """
        Check the validity of the arguments
        """
        self.param = {'quantity': [self.quantity, str],
                      'inter': [self.inter, str],
                      'ReIm': [self.ReIm, str],
                      'format_inp': [self.format_input, str],
                      'folder_inp': [self.folder, str],
                      'time': [self.time, int],
                      'time_num': [self.time_num, int],
                      'tem_iter': [self.tem_iter, str],
                      'species': [self.species, str],
                      'movie': [self.movie, int],
                      'tem_input': [self.tem_input, dict],
                      'exportData': [self.exportData, dict]}

        fields = {'quantity': ['intensity', 'efield', 'potential', 'dens',
                               'polarizability', 'spectrum', 'E_loss'],
                  'ReIm': ['re', 'im'],
                  'format_input': ['txt', 'hdf5'],
                  'tem_iter': ['tem', 'iter']}

        for keys, values in self.param.items():
            if not isinstance(values[0], values[1]):
                raise ValueError('Error: input ' + keys +
                                 ' not right type, must be ' + str(values[1]))

            if keys in fields.keys():
                if values[0] not in fields[keys]:
                    raise ValueError(
                        keys + ' can be only: ' + str(fields[keys]))

                if (values[0] == 'spectrum' or values[0] ==
                        'E_loss') and self.tem_iter != 'tem':
                    raise ValueError('spectrum only with tem')

                if values[0] == 'polarizability' and self.tem_iter != 'iter':
                    raise ValueError('polarizability only with iter')


class MBPT_LCAO_Properties_figure:
    """
    class that define the caracteristic of your Figures.
    Parameters:
    -----------
    self.fontsize (float, default: 30): fontsize of the labels
    self.axis_size (list of float, default: [20, 20]): fontsize of the tickle
    self.folder (string, default: 'images/'): folders where
                                              are save the pictures
    self.figx = 16
    self.figy = 12
    self.figsize (tuple, default: (self.figx, self.figy)): size of
                                        the figure (width, heigth)
    self.fatoms (string, default:'domiprod_atom2coord.xyz'): name of the
                                        file for the atomic positions
    self.plot_atoms (bolleen, default: True): plotting atoms or not
    self.color_arrow = 'red'
    self.ft_arrow_label = 30
    self.arrow_label = r'$E_{ext}$'
    self.title = 'none'
    self.linewidth = 3
    self.linecolor = 'red'
    self.plan (string, default: 'z'): plan than one wish to plot, can be
                                      x
                                      y
                                      z
    self.dynamic (int, default: 0): available only for Mayavi plot
                                        make the plot rotating, only 0 or 1
    self.show (int, default: 1): show the plot, 0 or 1
    self.plan_coord (float, default: 0.0): coordinate of the plan than one wish
                                          to plot in Ang
    self.output (string, default: 'pdf'): format of the output file, can be
                                        pdf
                                        png
                                        ps
                                        eps
                                        svg
    self.animation (int, default: 0): save a movie of the animation made
                                      by self.dynamic only available with
                                      Mayavi plot and if self.dynamic=1
    self.Edir (int, default: 0): plot direction of the E field, 0 or 1
    self.coord_Ef (2D numpy array default:'default') plotting
                                            coordinate E field??
    self.plot (string, default:'2D'): define the plotting method, can be
                                      1D
                                      2D
                                      3D
                                      Mayavi
    self.coord (1D numpy array, default: np.array([0.0, 0.0, 0.0]))
                                        coordinate for curve in 1D
    self.figname (string, default:'default.'): name of the figure
    """

    def __init__(self):
        import matplotlib.cm as cm

        self.fontsize = 30
        self.axis_size = [20, 20]
        self.folder = 'images/'
        self.figx = 16
        self.figy = 12
        self.figsize = (self.figx, self.figy)
        self.fatoms = 'domiprod_atom2coord.xyz'
        self.plot_atm = True
        self.color_arrow = 'red'
        self.ft_arrow_label = 30
        self.arrow_label = r'$E_{ext}$'
        self.title = None
        self.linewidth = 3
        self.linecolor = 'red'
        self.plan_coord = 0.0
        self.plan = 'z'
        self.dynamic = 0
        self.show = 1
        self.output = 'pdf'
        self.animation = 0
        self.Edir = 0
        self.coord_Ef = 'default'
        self.plot = '2D'
        self.coord = np.array([0.0, 0.0, 0.0])
        self.bohr_rad = 0.52917721
        self.figname = 'default'
        self.cmap = cm.jet
        self.units = 'au'
        # for the moment only the polarizability can be modify in nm**2
        self.vmin = None
        self.vmax = None
        # to save a mayavi scrennshot in order to perform subplot with
        # matplotlib
        self.mayavi_screenshot = 0

        self.maya_prop = {
            'extent_factor': 1.0,
            'figsize': (
                640,
                480),
            'contours': 3,
            'atoms_resolution': 8,
            'atoms_scale': 1,
            'fps': 20,
            'opacity': 0.5,
            'line_width': 2.0,
            'magnification': 1}
        self.maya_cam = {
            'distance': None,
            'azimuth': None,
            'elevation': None,
            'roll': None,
            'reset_roll': True,
            'figure': None,
            'focalpoint': 'auto'}



"""Structure optimization. """

import sys
import pickle
import time
from math import sqrt
from os.path import isfile

from ase.parallel import rank, barrier
from ase.io.trajectory import Trajectory
import collections


class Dynamics:
    """Base-class for all MD and structure optimization classes."""
    def __init__(self, atoms, logfile, trajectory, master=None):
        """Dynamics object.

        Parameters:

        atoms: Atoms object
            The Atoms object to operate on.

        logfile: file object or str
            If *logfile* is a string, a file with that name will be opened.
            Use '-' for stdout.

        trajectory: Trajectory object or str
            Attach trajectory object.  If *trajectory* is a string a
            Trajectory will be constructed.  Use *None* for no
            trajectory.

        master: boolean
            Defaults to None, which causes only rank 0 to save files.  If
            set to true,  this rank will save files.
        """

        self.atoms = atoms
        if master is None:
            master = rank == 0
        if not master:
            logfile = None
        elif isinstance(logfile, str):
            if logfile == '-':
                logfile = sys.stdout
            else:
                logfile = open(logfile, 'a')
        self.logfile = logfile

        self.observers = []
        self.nsteps = 0

        if trajectory is not None:
            if isinstance(trajectory, str):
                trajectory = Trajectory(trajectory, mode='w',
                                        atoms=atoms, master=master)
            self.attach(trajectory)

    def get_number_of_steps(self):
        return self.nsteps

    def insert_observer(self, function, position=0, interval=1,
                        *args, **kwargs):
        """Insert an observer."""
        if not isinstance(function, collections.Callable):
            function = function.write
        self.observers.insert(position, (function, interval, args, kwargs))

    def attach(self, function, interval=1, *args, **kwargs):
        """Attach callback function.

        If *interval > 0*, at every *interval* steps, call *function* with
        arguments *args* and keyword arguments *kwargs*.

        If *interval <= 0*, after step *interval*, call *function* with
        arguments *args* and keyword arguments *kwargs*.  This is
        currently zero indexed."""

        if hasattr(function, 'set_description'):
            d = self.todict()
            d.update(interval=interval)
            function.set_description(d)
        if not hasattr(function, '__call__'):
            function = function.write
        self.observers.append((function, interval, args, kwargs))

    def call_observers(self):
        for function, interval, args, kwargs in self.observers:
            call = False
            # Call every interval iterations
            if interval > 0:
                if (self.nsteps % interval) == 0:
                    call = True
            # Call only on iteration interval
            elif interval <= 0:
                if self.nsteps == abs(interval):
                    call = True
            if call:
                function(*args, **kwargs)


class Optimizer(Dynamics):
    """Base-class for all structure optimization classes."""
    def __init__(self, atoms, restart, logfile, trajectory, master=None,
                 force_consistent=False):
        """Structure optimizer object.

        Parameters:

        atoms: Atoms object
            The Atoms object to relax.

        restart: str
            Filename for restart file.  Default value is *None*.

        logfile: file object or str
            If *logfile* is a string, a file with that name will be opened.
            Use '-' for stdout.

        trajectory: Trajectory object or str
            Attach trajectory object.  If *trajectory* is a string a
            Trajectory will be constructed.  Use *None* for no
            trajectory.

        master: boolean
            Defaults to None, which causes only rank 0 to save files.  If
            set to true,  this rank will save files.

        force_consistent: boolean or None
            Use force-consistent energy calls (as opposed to the energy
            extrapolated to 0 K).  If force_consistent=None, uses
            force-consistent energies if available in the calculator, but
            falls back to force_consistent=False if not.
        """
        Dynamics.__init__(self, atoms, logfile, trajectory, master)
        self.force_consistent = force_consistent
        self.restart = restart

        if restart is None or not isfile(restart):
            self.initialize()
        else:
            self.read()
            barrier()

    def todict(self):
        description = {'type': 'optimization',
                       'optimizer': self.__class__.__name__}
        return description

    def initialize(self):
        pass

    def run(self, fmax=0.05, steps=100000000):
        """Run structure optimization algorithm.

        This method will return when the forces on all individual
        atoms are less than *fmax* or when the number of steps exceeds
        *steps*."""

        if self.force_consistent is None:
            self.set_force_consistent()
        self.fmax = fmax
        step = 0
        while step < steps:
            f = self.atoms.get_forces()
            self.log(f)
            self.call_observers()
            if self.converged(f):
                return
            self.step(f)
            self.nsteps += 1
            step += 1

    def converged(self, forces=None):
        """Did the optimization converge?"""
        if forces is None:
            forces = self.atoms.get_forces()
        if hasattr(self.atoms, 'get_curvature'):
            return ((forces**2).sum(axis=1).max() < self.fmax**2 and
                    self.atoms.get_curvature() < 0.0)
        return (forces**2).sum(axis=1).max() < self.fmax**2

    def log(self, forces):
        if self.logfile is None:
            return
        fmax = sqrt((forces**2).sum(axis=1).max())
        e = self.atoms.get_potential_energy(
            force_consistent=self.force_consistent)
        T = time.localtime()
        name = self.__class__.__name__
        if self.nsteps == 0:
            self.logfile.write(
                '%s  %4s %8s %15s  %12s\n' %
                (' '*len(name), 'Step', 'Time', 'Energy', 'fmax'))
            if self.force_consistent:
                self.logfile.write(
                    '*Force-consistent energies used in optimization.\n')
        self.logfile.write('%s:  %3d %02d:%02d:%02d %15.6f%1s %12.4f\n' %
                           (name, self.nsteps, T[3], T[4], T[5], e,
                            {1: '*', 0: ''}[self.force_consistent], fmax))
        self.logfile.flush()

    def dump(self, data):
        if rank == 0 and self.restart is not None:
            pickle.dump(data, open(self.restart, 'wb'), protocol=2)

    def load(self):
        return pickle.load(open(self.restart, 'rb'))

    def set_force_consistent(self):
        """Automatically sets force_consistent to True if force_consistent
        energies are supported by calculator; else False."""
        try:
            self.atoms.get_potential_energy(force_consistent=True)
        except KeyError:
            self.force_consistent = False
        else:
            self.force_consistent = True



from ase import Atoms
from ase.calculators.emt import EMT
from ase.md import VelocityVerlet
from ase.io import Trajectory

a = 3.6
b = a / 2
fcc = Atoms('Cu', positions=[(0, 0, 0)],
            cell=[(0, b, b), (b, 0, b), (b, b, 0)],
            pbc=1)
fcc *= (2, 1, 1)
fcc.set_calculator(EMT())
fcc.set_momenta([(0.9, 0.0, 0.0), (-0.9, 0, 0)])
md = VelocityVerlet(fcc, dt=0.1)
def f():
    print(fcc.get_potential_energy(), fcc.get_total_energy())
md.attach(f)
md.attach(Trajectory('Cu2.traj', 'w', fcc).write, interval=3)
md.run(steps=20)
fcc2 = Trajectory('Cu2.traj', 'r')[-1]




"""Berendsen NPT dynamics class."""

import numpy as np

from ase.md.nvtberendsen import NVTBerendsen
import ase.units as units


class NPTBerendsen(NVTBerendsen):
    """Berendsen (constant N, P, T) molecular dynamics.
    
    This dynamics scale the velocities and volumes to maintain a constant
    pressure and temperature.  The shape of the simulation cell is not
    altered, if that is desired use Inhomogenous_NPTBerendsen.

    Usage: NPTBerendsen(atoms, timestep, temperature, taut, pressure, taup)

    atoms
        The list of atoms.
        
    timestep
        The time step.

    temperature
        The desired temperature, in Kelvin.

    taut
        Time constant for Berendsen temperature coupling.

    fixcm
        If True, the position and momentum of the center of mass is
        kept unperturbed.  Default: True.

    pressure
        The desired pressure, in bar (1 bar = 1e5 Pa).

    taup
        Time constant for Berendsen pressure coupling.

    compressibility
        The compressibility of the material, water 4.57E-5 bar-1, in bar-1

    """

    def __init__(self, atoms, timestep, temperature, taut=0.5e3 * units.fs,
                 pressure=1.01325, taup=1e3 * units.fs,
                 compressibility=4.57e-5, fixcm=True,
                 trajectory=None, logfile=None, loginterval=1):

        NVTBerendsen.__init__(self, atoms, timestep, temperature, taut, fixcm,
                              trajectory,
                              logfile, loginterval)
        self.taup = taup
        self.pressure = pressure
        self.compressibility = compressibility

    def set_taup(self, taut):
        self.taut = taut

    def get_taup(self):
        return self.taut

    def set_pressure(self, pressure):
        self.pressure = pressure

    def get_pressure(self):
        return self.pressure

    def set_compressibility(self, compressibility):
        self.compressibility = compressibility

    def get_compressibility(self):
        return self.compressibility

    def set_timestep(self, timestep):
        self.dt = timestep

    def get_timestep(self):
        return self.dt

    def scale_positions_and_cell(self):
        """ Do the Berendsen pressure coupling,
        scale the atom position and the simulation cell."""

        taupscl = self.dt / self.taup
        stress = self.atoms.get_stress(voigt=False)
        old_pressure = -stress.trace() / 3 * 1e-5 / units.Pascal
        scl_pressure = (1.0 - taupscl * self.compressibility / 3.0 *
                        (self.pressure - old_pressure))

        #print "old_pressure", old_pressure
        #print "volume scaling by:", scl_pressure

        cell = self.atoms.get_cell()
        cell = scl_pressure * cell
        self.atoms.set_cell(cell, scale_atoms=True)

    def step(self, f):
        """ move one timestep forward using Berenden NPT molecular dynamics."""

        NVTBerendsen.scale_velocities(self)
        self.scale_positions_and_cell()

        #one step velocity verlet
        atoms = self.atoms
        p = self.atoms.get_momenta()
        p += 0.5 * self.dt * f

        if self.fixcm:
            # calculate the center of mass
            # momentum and subtract it
            psum = p.sum(axis=0) / float(len(p))
            p = p - psum

        self.atoms.set_positions(
            self.atoms.get_positions() +
            self.dt * p / self.atoms.get_masses()[:, np.newaxis])

        # We need to store the momenta on the atoms before calculating
        # the forces, as in a parallel Asap calculation atoms may
        # migrate during force calculations, and the momenta need to
        # migrate along with the atoms.  For the same reason, we
        # cannot use self.masses in the line above.

        self.atoms.set_momenta(p)
        f = self.atoms.get_forces()
        atoms.set_momenta(self.atoms.get_momenta() + 0.5 * self.dt * f)

        return f

        
class Inhomogeneous_NPTBerendsen(NPTBerendsen):
    """Berendsen (constant N, P, T) molecular dynamics.
    
    This dynamics scale the velocities and volumes to maintain a constant
    pressure and temperature.  The size of the unit cell is allowed to change
    independently in the three directions, but the angles remain constant.

    Usage: NPTBerendsen(atoms, timestep, temperature, taut, pressure, taup)

    atoms
        The list of atoms.
        
    timestep
        The time step.

    temperature
        The desired temperature, in Kelvin.

    taut
        Time constant for Berendsen temperature coupling.

    fixcm
        If True, the position and momentum of the center of mass is
        kept unperturbed.  Default: True.

    pressure
        The desired pressure, in bar (1 bar = 1e5 Pa).

    taup
        Time constant for Berendsen pressure coupling.

    compressibility
        The compressibility of the material, water 4.57E-5 bar-1, in bar-1

    """
    def scale_positions_and_cell(self):
        """ Do the Berendsen pressure coupling,
        scale the atom position and the simulation cell."""

        taupscl = self.dt * self.compressibility / self.taup / 3.0
        stress = - self.atoms.get_stress() * 1e-5 / units.Pascal
        if stress.shape == (6,):
            stress = stress[:3]
        elif stress.shape == (3, 3):
            stress = [stress[i][i] for i in range(3)]
        else:
            raise ValueError('Cannot use a stress tensor of shape ' +
                             str(stress.shape))
        pbc = self.atoms.get_pbc()
        scl_pressurex = 1.0 - taupscl * (self.pressure - stress[0]) * pbc[0]
        scl_pressurey = 1.0 - taupscl * (self.pressure - stress[1]) * pbc[1]
        scl_pressurez = 1.0 - taupscl * (self.pressure - stress[2]) * pbc[2]
        
        cell = self.atoms.get_cell()
        cell = np.array([scl_pressurex * cell[0],
                         scl_pressurey * cell[1],
                         scl_pressurez * cell[2]])
        self.atoms.set_cell(cell, scale_atoms=True)



from ase import Atoms
from ase.db import connect
import ase.db.app as app
c = connect('test.db', append=False)
plot = {'title': 'A test',
        'data': [{'label': 't1', 'x': 'x', 'y': 't1'},
                 {'label': 't2', 'style': 'y--',
                  'x': 'x', 'y': 't2'}],
        'xlabel': 'x',
        'ylabel': 'y'}
x = [0, 1, 2]
t1 = [1, 2, 0]
t2 = [[2, 3], [1, 1], [1, 0]]
c.write(Atoms('H2O'),
        foo='bar',
        data={'test': plot,
              'x': x,
              't1': t1,
              't2': t2})
app.db = c
app.app.testing = True
d = app.app.test_client().get('/')
print(d)
d = app.app.test_client().get('/id/1')
print(d)
d = app.app.test_client().get('/plot/test-1.png')
print(d, app.tmpdir)



from ase.test import NotAvailable
from ase.build import bulk
from ase.calculators.calculator import get_calculator


required = {'abinit': dict(ecut=200, toldfe=0.0001, chksymbreak=0),
            'aims': dict(sc_accuracy_rho=5.e-3),
            'elk': dict(tasks=0, rgkmax=5.0),
            'gpaw': dict(mode='pw'),
            'cp2k': dict(auto_write=True, uks=True, max_scf=1, cutoff=400)}


def run(name):
    Calculator = get_calculator(name)
    par = required.get(name, {})
    calc = Calculator(label=name, xc='LDA', kpts=1.0, **par)
    al = bulk('AlO', crystalstructure='rocksalt', a=4.5)
    al.calc = calc
    e = al.get_potential_energy()
    calc.set(xc='PBE', kpts=(2, 2, 2))
    epbe = al.get_potential_energy()
    print(e, epbe)
    calc = Calculator(name)
    print(calc.parameters, calc.results, calc.atoms)
    assert not calc.calculation_required(al, ['energy'])
    al = calc.get_atoms()
    print(al.get_potential_energy())
    label = 'dir/' + name + '-2'
    calc = Calculator(label=label, atoms=al, xc='LDA', kpts=1.0, **par)
    print(al.get_potential_energy())
    print(Calculator.read_atoms(label).get_potential_energy())

names = ['abinit', 'aims', 'elk', 'cp2k']
for name in names:
    try:
        run(name)
    except NotAvailable:
        pass



""" Class for handling interaction with the PBS queuing system."""
from ase.io import write
import os
from ase.io.trajectory import Trajectory
from subprocess import Popen, PIPE
import time


class PBSQueueRun(object):

    """ Class for communicating with the commonly used PBS queing system
         at a computer cluster.

        The user needs to supply a job file generator which takes
        as input a job name and the relative path to the traj
        file which is to be locally optimized. The function returns
        the job script as text.
        If the traj file is called f the job must write a file
        f[:-5] + '_done.traj' which is then read by this object.

       Parameters:

       data_connection: The DataConnection object.
       tmp_folder: Temporary folder for all calculations
       job_prefix: Prefix of the job submitted. This identifier is used
       to determine how many jobs are currently running.
       n_simul: The number of simultaneous jobs to keep in the queuing system.
       job_template_generator: The function generating the job file.
       This function should return the content of the job file as a
       string.
       qsub_command: The name of the qsub command (default qsub).
       qstat_command: The name of the qstat command (default qstat).
    """
    def __init__(self, data_connection, tmp_folder, job_prefix,
                 n_simul, job_template_generator,
                 qsub_command='qsub', qstat_command='qstat',
                 find_neighbors=None, perform_parametrization=None):
        self.dc = data_connection
        self.job_prefix = job_prefix
        self.n_simul = n_simul
        self.job_template_generator = job_template_generator
        self.qsub_command = qsub_command
        self.qstat_command = qstat_command
        self.tmp_folder = tmp_folder
        self.find_neighbors = find_neighbors
        self.perform_parametrization = perform_parametrization
        self.__cleanup__()

    def relax(self, a):
        """ Add a structure to the queue. This method does not fail
            if sufficient jobs are already running, but simply
            submits the job. """
        self.__cleanup__()
        self.dc.mark_as_queued(a)
        if not os.path.isdir(self.tmp_folder):
            os.mkdir(self.tmp_folder)
        fname = '{0}/cand{1}.traj'.format(self.tmp_folder,
                                          a.info['confid'])
        write(fname, a)
        job_name = '{0}_{1}'.format(self.job_prefix, a.info['confid'])
        f = open('tmp_job_file.job', 'w')
        f.write(self.job_template_generator(job_name, fname))
        f.close()
        os.system('{0} tmp_job_file.job'.format(self.qsub_command))

    def enough_jobs_running(self):
        """ Determines if sufficient jobs are running. """
        return self.number_of_jobs_running() >= self.n_simul

    def number_of_jobs_running(self):
        """ Determines how many jobs are running. The user
            should use this or the enough_jobs_running method
            to verify that a job needs to be started before
            calling the relax method."""
        self.__cleanup__()
        p = Popen(['`which {0}` -u `whoami`'.format(self.qstat_command)],
                  shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE,
                  close_fds=True)
        fout = p.stdout
        lines = fout.readlines()
        n_running = 0
        for l in lines:
            if l.find(self.job_prefix) != -1:
                n_running += 1
        return n_running

    def __cleanup__(self):
        """ Tries to load in structures previously
            submitted to the queing system. """
        confs = self.dc.get_all_candidates_in_queue()
        for c in confs:
            fdone = '{0}/cand{1}_done.traj'.format(self.tmp_folder,
                                                   c)
            if os.path.isfile(fdone) and os.path.getsize(fdone) > 0:
                try:
                    a = []
                    niter = 0
                    while len(a) == 0 and niter < 5:
                        t = Trajectory(fdone, 'r')
                        a = [ats for ats in t]
                        if len(a) == 0:
                            time.sleep(1.)
                        niter += 1
                    if len(a) == 0:
                        txt = 'Could not read candidate ' + \
                            '{0} from the filesystem'.format(c)
                        raise IOError(txt)
                    a = a[-1]
                    a.info['confid'] = c
                    self.dc.add_relaxed_step(
                        a,
                        find_neighbors=self.find_neighbors,
                        perform_parametrization=self.perform_parametrization)
                except IOError as e:
                    print(e)



# -*- coding: utf-8 -*-

"""This module defines an ASE interface to CP2K.

http://www.cp2k.org
Author: Ole Schuett <ole.schuett@mat.ethz.ch>
"""

from __future__ import print_function

import os
import os.path
from warnings import warn
from subprocess import Popen, PIPE
import numpy as np
import ase.io
from ase.units import Rydberg
from ase.calculators.calculator import Calculator, all_changes, Parameters


class CP2K(Calculator):
    """ASE-Calculator for CP2K.

    CP2K is a program to perform atomistic and molecular simulations of solid
    state, liquid, molecular, and biological systems. It provides a general
    framework for different methods such as e.g., density functional theory
    (DFT) using a mixed Gaussian and plane waves approach (GPW) and classical
    pair and many-body potentials.

    CP2K is freely available under the GPL license.
    It is written in Fortran 2003 and can be run efficiently in parallel.

    Check http://www.cp2k.org about how to obtain and install CP2K.
    Make sure that you also have the CP2K-shell available, since it is required
    by the CP2K-calulator.

    The CP2K-calculator relies on the CP2K-shell. The CP2K-shell was originally
    designed for interactive sessions. When a calculator object is
    instantiated, it launches a CP2K-shell as a subprocess in the background
    and communications with it through stdin/stdout pipes. This has the
    advantage that the CP2K process is kept alive for the whole lifetime of
    the calculator object, i.e. there is no startup overhead for a sequence
    of energy evaluations. Furthermore, the usage of pipes avoids slow file-
    system I/O. This mechanism even works for MPI-parallelized runs, because
    stdin/stdout of the first rank are forwarded by the MPI-environment to the
    mpiexec-process.

    The command used by the calculator to launch the CP2K-shell is
    ``cp2k_shell``. To run a parallelized simulation use something like this:

    >>> CP2K.command="env OMP_NUM_THREADS=2 mpiexec -np 4 cp2k_shell.psmp"


    Arguments:

    auto_write: bool
        Flag to enable the auto-write mode. If enabled the
        ``write()`` routine is called after every
        calculation, which mimics the behavior of the
        ``FileIOCalculator``. Default is ``False``.
    basis_set: str
        Name of the basis set to be use.
        The default is ``DZVP-MOLOPT-SR-GTH``.
    basis_set_file: str
        Filename of the basis set file.
        Default is ``BASIS_MOLOPT``.
        Set the environment variable $CP2K_DATA_DIR
        to enabled automatic file discovered.
    charge: float
        The total charge of the system.  Default is ``0``.
    command: str
        The command used to launch the CP2K-shell.
        If ``command`` is not passed as an argument to the
        constructor, the class-variable ``CP2K.command``,
        and then the environment variabel
        ``$ASE_CP2K_COMMAND`` are checked.
        Eventually, ``cp2k_shell`` is used as default.
    cutoff: float
        The cutoff of the finest grid level.  Default is ``400 * Rydberg``.
    debug: bool
        Flag to enable debug mode. This will print all
        communication between the CP2K-shell and the
        CP2K-calculator. Default is ``False``.
    force_eval_method: str
        The method CP2K uses to evaluate energies and forces.
        The default is ``Quickstep``, which is CP2K's
        module for electronic structure methods like DFT.
    inp: str
        CP2K input template. If present, the calculator will
        augment the template, e.g. with coordinates, and use
        it to launch CP2K. Hence, this generic mechanism
        gives access to all features of CP2K.
        Note, that most keywords accept ``None`` to disable the generation
        of the corresponding input section.
    max_scf: int
        Maximum number of SCF iteration to be performed for
        one optimization. Default is ``50``.
    poisson_solver: str
        The poisson solver to be used. Currently, the only supported
        values are ``auto`` and ``None``. Default is ``auto``.
    potential_file: str
        Filename of the pseudo-potential file.
        Default is ``POTENTIAL``.
        Set the environment variable $CP2K_DATA_DIR
        to enabled automatic file discovered.
    pseudo_potential: str
        Name of the pseudo-potential to be use.
        Default is ``auto``. This tries to infer the
        potential from the employed XC-functional,
        otherwise it falls back to ``GTH-PBE``.
    stress_tensor: bool
        Indicates whether the analytic stress-tensor should be calculated.
        Default is ``True``.
    uks: bool
        Requests an unrestricted Kohn-Sham calculations.
        This is need for spin-polarized systems, ie. with an
        odd number of electrons. Default is ``False``.
    xc: str
        Name of exchange and correlation functional.
        Accepts all functions supported by CP2K itself or libxc.
        Default is ``LDA``.

    """

    implemented_properties = ['energy', 'forces', 'stress']
    command = None

    default_parameters = dict(
        auto_write=False,
        basis_set='DZVP-MOLOPT-SR-GTH',
        basis_set_file='BASIS_MOLOPT',
        charge=0,
        cutoff=400 * Rydberg,
        force_eval_method="Quickstep",
        inp='',
        max_scf=50,
        potential_file='POTENTIAL',
        pseudo_potential='auto',
        stress_tensor=True,
        uks=False,
        poisson_solver='auto',
        xc='LDA')

    def __init__(self, restart=None, ignore_bad_restart_file=False,
                 label='cp2k', atoms=None, command=None,
                 debug=False, **kwargs):
        """Construct CP2K-calculator object."""

        self._debug = debug
        self._force_env_id = None
        self._shell = None
        self.label = None
        self.parameters = None
        self.results = None
        self.atoms = None

        # Several places are check to determine self.command
        if command is not None:
            self.command = command
        elif CP2K.command is not None:
            self.command = CP2K.command
        elif 'ASE_CP2K_COMMAND' in os.environ:
            self.command = os.environ['ASE_CP2K_COMMAND']
        else:
            self.command = 'cp2k_shell'  # default

        Calculator.__init__(self, restart, ignore_bad_restart_file,
                            label, atoms, **kwargs)

        self._shell = Cp2kShell(self.command, self._debug)

        if restart is not None:
            try:
                self.read(restart)
            except:
                if ignore_bad_restart_file:
                    self.reset()
                else:
                    raise

    def __del__(self):
        """Release force_env and terminate cp2k_shell child process"""
        if self._shell:
            self._release_force_env()
            del(self._shell)

    def set(self, **kwargs):
        """Set parameters like set(key1=value1, key2=value2, ...)."""
        changed_parameters = Calculator.set(self, **kwargs)
        if changed_parameters:
            self.reset()

    def write(self, label):
        'Write atoms, parameters and calculated results into restart files.'
        if self._debug:
            print("Writting restart to: ", label)
        self.atoms.write(label + '_restart.traj')
        self.parameters.write(label + '_params.ase')
        open(label + '_results.ase', 'w').write(repr(self.results))

    def read(self, label):
        'Read atoms, parameters and calculated results from restart files.'
        self.atoms = ase.io.read(label + '_restart.traj')
        self.parameters = Parameters.read(label + '_params.ase')
        results_txt = open(label + '_results.ase').read()
        self.results = eval(results_txt, {'array': np.array})

    def calculate(self, atoms=None, properties=None,
                  system_changes=all_changes):
        """Do the calculation."""

        if not properties:
            properties = ['energy']
        Calculator.calculate(self, atoms, properties, system_changes)

        if self._debug:
            print("system_changes:", system_changes)

        if 'numbers' in system_changes:
            self._release_force_env()

        if self._force_env_id is None:
            self._create_force_env()

        # enable eV and Angstrom as units
        self._shell.send('UNITS_EV_A')
        assert self._shell.recv() == '* READY'

        n_atoms = len(self.atoms)
        if 'cell' in system_changes:
            cell = self.atoms.get_cell()
            self._shell.send('SET_CELL %d' % self._force_env_id)
            for i in range(3):
                self._shell.send('%.18e %.18e %.18e' % tuple(cell[i, :]))
            assert self._shell.recv() == '* READY'

        if 'positions' in system_changes:
            self._shell.send('SET_POS %d' % self._force_env_id)
            self._shell.send('%d' % (3 * n_atoms))
            for pos in self.atoms.get_positions():
                self._shell.send('%.18e %.18e %.18e' % tuple(pos))
            self._shell.send('*END')
            assert float(self._shell.recv()) >= 0  # max change -> ignore
            assert self._shell.recv() == '* READY'

        self._shell.send('EVAL_EF %d' % self._force_env_id)
        assert self._shell.recv() == '* READY'

        self._shell.send('GET_E %d' % self._force_env_id)
        self.results['energy'] = float(self._shell.recv())
        assert self._shell.recv() == '* READY'

        forces = np.zeros(shape=(n_atoms, 3))
        self._shell.send('GET_F %d' % self._force_env_id)
        assert int(self._shell.recv()) == 3 * n_atoms
        for i in range(n_atoms):
            line = self._shell.recv()
            forces[i, :] = [float(x) for x in line.split()]
        assert self._shell.recv() == '* END'
        assert self._shell.recv() == '* READY'
        self.results['forces'] = forces

        self._shell.send('GET_STRESS %d' % self._force_env_id)
        line = self._shell.recv()
        assert self._shell.recv() == '* READY'

        stress = np.array([float(x) for x in line.split()]).reshape(3, 3)
        assert np.all(stress == np.transpose(stress))   # should be symmetric
        # Convert 3x3 stress tensor to Voigt form as required by ASE
        stress = np.array([stress[0, 0], stress[1, 1], stress[2, 2],
                           stress[1, 2], stress[0, 2], stress[0, 1]])
        self.results['stress'] = -1.0 * stress  # cp2k uses the opposite sign

        if self.parameters.auto_write:
            self.write(self.label)

    def _create_force_env(self):
        """Instantiates a new force-environment"""
        assert self._force_env_id is None
        label_dir = os.path.dirname(self.label)
        if len(label_dir) > 0 and not os.path.exists(label_dir):
            print('Creating directory: ' + label_dir)
            os.makedirs(label_dir)  # cp2k expects dirs to exist

        inp = self._generate_input()
        inp_fn = self.label + '.inp'
        out_fn = self.label + '.out'
        self._write_file(inp_fn, inp)
        self._shell.send('LOAD %s %s' % (inp_fn, out_fn))
        self._force_env_id = int(self._shell.recv())
        assert self._force_env_id > 0
        assert self._shell.recv() == '* READY'

    def _write_file(self, fn, content):
        """Write content to a file"""
        if self._debug:
            print('Writting to file: ' + fn)
            print(content)
        if self._shell.version < 2.0:
            f = open(fn, 'w')
            f.write(content)
            f.close()
        else:
            lines = content.split('\n')
            if self._shell.version < 2.1:
                lines = [l.strip() for l in lines]  # save chars
            self._shell.send('WRITE_FILE')
            self._shell.send(fn)
            self._shell.send('%d' % len(lines))
            for line in lines:
                self._shell.send(line)
            self._shell.send('*END')
            assert self._shell.recv() == '* READY'

    def _release_force_env(self):
        """Destroys the current force-environment"""
        if self._force_env_id:
            if self._shell.isready:
                self._shell.send('DESTROY %d' % self._force_env_id)
                assert self._shell.recv() == '* READY'
            else:
                msg = "CP2K-shell not ready, could not release force_env."
                warn(msg, RuntimeWarning)
            self._force_env_id = None

    def _generate_input(self):
        """Generates a CP2K input file"""
        p = self.parameters
        root = parse_input(p.inp)
        root.add_keyword('GLOBAL', 'PROJECT ' + self.label)
        if p.force_eval_method:
            root.add_keyword('FORCE_EVAL', 'METHOD ' + p.force_eval_method)
        if p.stress_tensor:
            root.add_keyword('FORCE_EVAL', 'STRESS_TENSOR ANALYTICAL')
            root.add_keyword('FORCE_EVAL/PRINT/STRESS_TENSOR',
                             '_SECTION_PARAMETERS_ ON')
        if p.basis_set_file:
            root.add_keyword('FORCE_EVAL/DFT',
                             'BASIS_SET_FILE_NAME ' + p.basis_set_file)
        if p.potential_file:
            root.add_keyword('FORCE_EVAL/DFT',
                             'POTENTIAL_FILE_NAME ' + p.potential_file)
        if p.cutoff:
            root.add_keyword('FORCE_EVAL/DFT/MGRID',
                             'CUTOFF [eV] %.18e' % p.cutoff)
        if p.max_scf:
            root.add_keyword('FORCE_EVAL/DFT/SCF', 'MAX_SCF %d' % p.max_scf)
            root.add_keyword('FORCE_EVAL/DFT/LS_SCF', 'MAX_SCF %d' % p.max_scf)

        if p.xc:
            if p.xc.startswith("XC_"):
                root.add_keyword('FORCE_EVAL/DFT/XC/XC_FUNCTIONAL/LIBXC',
                                 'FUNCTIONAL ' + p.xc)
            else:
                root.add_keyword('FORCE_EVAL/DFT/XC/XC_FUNCTIONAL',
                                 '_SECTION_PARAMETERS_ ' + p.xc)

        if p.uks:
            root.add_keyword('FORCE_EVAL/DFT', 'UNRESTRICTED_KOHN_SHAM ON')

        if p.charge and p.charge != 0:
            root.add_keyword('FORCE_EVAL/DFT', 'CHARGE %d' % p.charge)

        # add Poisson solver if needed
        if p.poisson_solver == 'auto' and not any(self.atoms.get_pbc()):
            root.add_keyword('FORCE_EVAL/DFT/POISSON', 'PERIODIC NONE')
            root.add_keyword('FORCE_EVAL/DFT/POISSON', 'PSOLVER  MT')

        # write coords
        syms = self.atoms.get_chemical_symbols()
        atoms = self.atoms.get_positions()
        for elm, pos in zip(syms, atoms):
            line = '%s %.18e %.18e %.18e' % (elm, pos[0], pos[1], pos[2])
            root.add_keyword('FORCE_EVAL/SUBSYS/COORD', line, unique=False)

        # write cell
        pbc = ''.join([a for a, b in zip('XYZ', self.atoms.get_pbc()) if b])
        if len(pbc) == 0:
            pbc = 'NONE'
        root.add_keyword('FORCE_EVAL/SUBSYS/CELL', 'PERIODIC ' + pbc)
        c = self.atoms.get_cell()
        for i, a in enumerate('ABC'):
            line = '%s %.18e %.18e %.18e' % (a, c[i, 0], c[i, 1], c[i, 2])
            root.add_keyword('FORCE_EVAL/SUBSYS/CELL', line)

        # determine pseudo-potential
        potential = p.pseudo_potential
        if p.pseudo_potential == 'auto':
            if p.xc and p.xc.upper() in ('LDA', 'PADE', 'BP', 'BLYP', 'PBE',):
                potential = 'GTH-' + p.xc.upper()
            else:
                msg = 'No matching pseudo potential found, using GTH-PBE'
                warn(msg, RuntimeWarning)
                potential = 'GTH-PBE'  # fall back

        # write atomic kinds
        subsys = root.get_subsection('FORCE_EVAL/SUBSYS').subsections
        kinds = dict([(s.params, s) for s in subsys if s.name == "KIND"])
        for elem in set(self.atoms.get_chemical_symbols()):
            if elem not in kinds.keys():
                s = InputSection(name='KIND', params=elem)
                subsys.append(s)
                kinds[elem] = s
            if p.basis_set:
                kinds[elem].keywords.append('BASIS_SET ' + p.basis_set)
            if potential:
                kinds[elem].keywords.append('POTENTIAL ' + potential)

        output_lines = ['!!! Generated by ASE !!!'] + root.write()
        return '\n'.join(output_lines)


class Cp2kShell(object):
    """Wrapper for CP2K-shell child-process"""

    def __init__(self, command, debug):
        """Construct CP2K-shell object"""

        self.isready = False
        self.version = 1.0  # assume oldest possible version until verified
        self._child = None
        self._debug = debug

        # launch cp2k_shell child process
        assert 'cp2k_shell' in command
        if self._debug:
            print(command)
        self._child = Popen(command, shell=True, universal_newlines=True,
                            stdin=PIPE, stdout=PIPE, bufsize=1)
        assert self.recv() == '* READY'

        # check version of shell
        self.send('VERSION')
        shell_version = self.recv().rsplit(":", 1)
        assert self.recv() == '* READY'
        assert shell_version[0] == "CP2K Shell Version"
        self.version = float(shell_version[1])
        assert self.version >= 1.0

        # enable harsh mode, stops on any error
        self.send('HARSH')
        assert self.recv() == '* READY'

    def __del__(self):
        """Terminate cp2k_shell child process"""
        if self.isready:
            self.send('EXIT')
            assert self._child.wait() == 0  # child process exited properly?
        else:
            warn("CP2K-shell not ready, sending SIGTERM.", RuntimeWarning)
            self._child.terminate()
        self._child = None
        self.version = None
        self.isready = False

    def send(self, line):
        """Send a line to the cp2k_shell"""
        assert self._child.poll() is None  # child process still alive?
        if self._debug:
            print('Sending: ' + line)
        if self.version < 2.1 and len(line) >= 80:
            raise Exception('Buffer overflow, upgrade CP2K to r16779 or later')
        assert(len(line) < 800)  # new input buffer size
        self.isready = False
        self._child.stdin.write(line + '\n')

    def recv(self):
        """Receive a line from the cp2k_shell"""
        assert self._child.poll() is None  # child process still alive?
        line = self._child.stdout.readline().strip()
        if self._debug:
            print('Received: ' + line)
        self.isready = line == '* READY'
        return line


class InputSection(object):
    """Represents a section of a CP2K input file"""
    def __init__(self, name, params=None):
        self.name = name.upper()
        self.params = params
        self.keywords = []
        self.subsections = []

    def write(self):
        """Outputs input section as string"""
        output = []
        for k in self.keywords:
            output.append(k)
        for s in self.subsections:
            if s.params:
                output.append('&%s %s' % (s.name, s.params))
            else:
                output.append('&%s' % s.name)
            for l in s.write():
                output.append('   %s' % l)
            output.append('&END %s' % s.name)
        return output

    def add_keyword(self, path, line, unique=True):
        """Adds a keyword to section."""
        parts = path.upper().split('/', 1)
        candidates = [s for s in self.subsections if s.name == parts[0]]
        if len(candidates) == 0:
            s = InputSection(name=parts[0])
            self.subsections.append(s)
            candidates = [s]
        elif len(candidates) != 1:
            raise Exception('Multiple %s sections found ' % parts[0])

        key = line.split()[0].upper()
        if len(parts) > 1:
            candidates[0].add_keyword(parts[1], line, unique)
        elif key == '_SECTION_PARAMETERS_':
            if candidates[0].params is not None:
                msg = 'Section parameter of section %s already set' % parts[0]
                raise Exception(msg)
            candidates[0].params = line.split(' ', 1)[1].strip()
        else:
            old_keys = [k.split()[0].upper() for k in candidates[0].keywords]
            if unique and key in old_keys:
                msg = 'Keyword %s already present in section %s'
                raise Exception(msg % (key, parts[0]))
            candidates[0].keywords.append(line)

    def get_subsection(self, path):
        """Finds a subsection"""
        parts = path.upper().split('/', 1)
        candidates = [s for s in self.subsections if s.name == parts[0]]
        if len(candidates) > 1:
            raise Exception('Multiple %s sections found ' % parts[0])
        if len(candidates) == 0:
            s = InputSection(name=parts[0])
            self.subsections.append(s)
            candidates = [s]
        if len(parts) == 1:
            return candidates[0]
        return candidates[0].get_subsection(parts[1])


def parse_input(inp):
    """Parses the given CP2K input string"""
    root_section = InputSection('CP2K_INPUT')
    section_stack = [root_section]

    for line in inp.split('\n'):
        line = line.split('!', 1)[0].strip()
        if len(line) == 0:
            continue

        if line.upper().startswith('&END'):
            s = section_stack.pop()
        elif line[0] == '&':
            parts = line.split(' ', 1)
            name = parts[0][1:]
            if len(parts) > 1:
                s = InputSection(name=name, params=parts[1].strip())
            else:
                s = InputSection(name=name)
            section_stack[-1].subsections.append(s)
            section_stack.append(s)
        else:
            section_stack[-1].keywords.append(line)

    return root_section



from __future__ import print_function
import warnings

from ase import __version__
from ase.calculators.singlepoint import SinglePointCalculator, all_properties
from ase.constraints import dict2constraint
from ase.calculators.calculator import PropertyNotImplementedError
from ase.atoms import Atoms
from ase.io.jsonio import encode, decode
from ase.io.pickletrajectory import PickleTrajectory
from ase.parallel import world

__all__ = ['Trajectory', 'PickleTrajectory']


def Trajectory(filename, mode='r', atoms=None, properties=None, master=None):
    """A Trajectory can be created in read, write or append mode.

    Parameters:

    filename: str
        The name of the file.  Traditionally ends in .traj.
    mode: str
        The mode.  'r' is read mode, the file should already exist, and
        no atoms argument should be specified.
        'w' is write mode.  The atoms argument specifies the Atoms
        object to be written to the file, if not given it must instead
        be given as an argument to the write() method.
        'a' is append mode.  It acts as write mode, except that
        data is appended to a preexisting file.
    atoms: Atoms object
        The Atoms object to be written in write or append mode.
    properties: list of str
        If specified, these calculator properties are saved in the
        trajectory.  If not specified, all supported quantities are
        saved.  Possible values: energy, forces, stress, dipole,
        charges, magmom and magmoms.
    master: bool
        Controls which process does the actual writing. The
        default is that process number 0 does this.  If this
        argument is given, processes where it is True will write.

    The atoms, properties and master arguments are ignores in read mode.
    """
    if mode == 'r':
        return TrajectoryReader(filename)
    return TrajectoryWriter(filename, mode, atoms, properties, master=master)


class TrajectoryWriter:
    """Writes Atoms objects to a .traj file."""
    def __init__(self, filename, mode='w', atoms=None, properties=None,
                 extra=[], master=None):
        """A Trajectory writer, in write or append mode.

        Parameters:

        filename: str
            The name of the file.  Traditionally ends in .traj.
        mode: str
            The mode.  'r' is read mode, the file should already exist, and
            no atoms argument should be specified.
            'w' is write mode.  The atoms argument specifies the Atoms
            object to be written to the file, if not given it must instead
            be given as an argument to the write() method.
            'a' is append mode.  It acts as write mode, except that
            data is appended to a preexisting file.
        atoms: Atoms object
            The Atoms object to be written in write or append mode.
        properties: list of str
            If specified, these calculator properties are saved in the
            trajectory.  If not specified, all supported quantities are
            saved.  Possible values: energy, forces, stress, dipole,
            charges, magmom and magmoms.
        master: bool
            Controls which process does the actual writing. The
            default is that process number 0 does this.  If this
            argument is given, processes where it is True will write.
        """
        if master is None:
            master = (world.rank == 0)
        self.master = master
        self.atoms = atoms
        self.properties = properties

        self.description = {}
        self.numbers = None
        self.pbc = None
        self.masses = None
        self._open(filename, mode)

    def set_description(self, description):
        self.description.update(description)

    def _open(self, filename, mode):
        from ase.io.ulm import ulmopen, DummyWriter
        if mode not in 'aw':
            raise ValueError('mode must be "w" or "a".')
        if self.master:
            self.backend = ulmopen(filename, mode, tag='ASE-Trajectory')
            if len(self.backend) > 0:
                r = ulmopen(filename)
                self.numbers = r.numbers
                self.pbc = r.pbc
        else:
            self.backend = DummyWriter()

    def write(self, atoms=None, **kwargs):
        """Write the atoms to the file.

        If the atoms argument is not given, the atoms object specified
        when creating the trajectory object is used.

        Use keyword arguments to add extra properties::

            writer.write(atoms, energy=117, dipole=[0, 0, 1.0])
        """
        b = self.backend

        if atoms is None:
            atoms = self.atoms

        if hasattr(atoms, 'interpolate'):
            # seems to be a NEB
            neb = atoms
            assert not neb.parallel or world.size == 1
            for image in neb.images:
                self.write(image)
            return
        while hasattr(atoms, 'atoms_for_saving'):
            # Seems to be a Filter or similar, instructing us to
            # save the original atoms.
            atoms = atoms.atoms_for_saving

        if len(b) == 0:
            b.write(version=1, ase_version=__version__)
            if self.description:
                b.write(description=self.description)
            # Atomic numbers and periodic boundary conditions are only
            # written once - in the header.  Store them here so that we can
            # check that they are the same for all images:
            self.numbers = atoms.get_atomic_numbers()
            self.pbc = atoms.get_pbc()
        else:
            if (atoms.pbc != self.pbc).any():
                raise ValueError('Bad periodic boundary conditions!')
            elif len(atoms) != len(self.numbers):
                raise ValueError('Bad number of atoms!')
            elif (atoms.numbers != self.numbers).any():
                raise ValueError('Bad atomic numbers!')

        write_atoms(b, atoms, write_header=(len(b) == 0))

        calc = atoms.get_calculator()

        if calc is None and len(kwargs) > 0:
            calc = SinglePointCalculator(atoms)

        if calc is not None:
            if not hasattr(calc, 'get_property'):
                calc = OldCalculatorWrapper(calc)
            c = b.child('calculator')
            c.write(name=calc.name)
            if hasattr(calc, 'todict'):
                d = calc.todict()
                if d:
                    c.write(parameters=d)
            for prop in all_properties:
                if prop in kwargs:
                    x = kwargs[prop]
                else:
                    if self.properties is not None:
                        if prop in self.properties:
                            x = calc.get_property(prop, atoms)
                        else:
                            x = None
                    else:
                        try:
                            x = calc.get_property(prop, atoms,
                                                  allow_calculation=False)
                        except (PropertyNotImplementedError, KeyError):
                            # KeyError is needed for Jacapo.
                            x = None
                if x is not None:
                    if prop in ['stress', 'dipole']:
                        x = x.tolist()
                    c.write(prop, x)

        info = {}
        for key, value in atoms.info.items():
            try:
                encode(value)
            except TypeError:
                warnings.warn('Skipping "{0}" info.'.format(key))
            else:
                info[key] = value
        if info:
            b.write(info=info)

        b.sync()

    def close(self):
        """Close the trajectory file."""
        self.backend.close()

    def __len__(self):
        return world.sum(len(self.backend))


class TrajectoryReader:
    """Reads Atoms objects from a .traj file."""
    def __init__(self, filename):
        """A Trajectory in read mode.

        The filename traditionally ends in .traj.
        """

        self.numbers = None
        self.pbc = None
        self.masses = None

        self._open(filename)

    def _open(self, filename):
        from ase.io.ulm import ulmopen, InvalidULMFileError
        try:
            self.backend = ulmopen(filename, 'r')
        except InvalidULMFileError:
            raise RuntimeError('This is not a valid ASE trajectory file. '
                               'If this is an old-format (version <3.9) '
                               'PickleTrajectory file you can convert it '
                               'with ase.io.trajectory.convert("%s") '
                               'or:\n\n $ python -m ase.io.trajectory %s'
                               % (filename, filename))
        self._read_header()

    def _read_header(self):
        b = self.backend
        if b.get_tag() != 'ASE-Trajectory':
            raise IOError('This is not a trajectory file!')

        if len(b) > 0:
            self.pbc = b.pbc
            self.numbers = b.numbers
            self.masses = b.get('masses')
            self.constraints = b.get('constraints', '[]')
            self.description = b.get('description')
            self.version = b.version
            self.ase_version = b.get('ase_version')

    def close(self):
        """Close the trajectory file."""
        self.backend.close()

    def __getitem__(self, i=-1):
        b = self.backend[i]
        atoms = read_atoms(b, header=[self.pbc, self.numbers, self.masses,
                                      self.constraints])
        if 'calculator' in b:
            results = {}
            c = b.calculator
            for prop in all_properties:
                if prop in c:
                    results[prop] = c.get(prop)
            calc = SinglePointCalculator(atoms, **results)
            calc.name = b.calculator.name
            atoms.set_calculator(calc)
        return atoms

    def __len__(self):
        return len(self.backend)

    def __iter__(self):
        for i in range(len(self)):
            yield self[i]


def read_atoms(backend, header=None):
    b = backend
    if header:
        pbc, numbers, masses, constraints = header
    else:
        pbc = b.pbc
        numbers = b.numbers
        masses = b.get('masses')
        constraints = b.get('constraints', '[]')

    atoms = Atoms(positions=b.positions,
                  numbers=numbers,
                  cell=b.cell,
                  masses=masses,
                  pbc=pbc,
                  info=b.get('info'),
                  constraint=[dict2constraint(d)
                              for d in decode(constraints)],
                  momenta=b.get('momenta'),
                  magmoms=b.get('magmoms'),
                  charges=b.get('charges'),
                  tags=b.get('tags'))
    return atoms


def write_atoms(backend, atoms, write_header=True):
    b = backend

    if write_header:
        b.write(pbc=atoms.pbc.tolist(),
                numbers=atoms.numbers)
        if atoms.constraints:
            if all(hasattr(c, 'todict') for c in atoms.constraints):
                b.write(constraints=encode(atoms.constraints))

        if atoms.has('masses'):
            b.write(masses=atoms.get_masses())

    b.write(positions=atoms.get_positions(),
            cell=atoms.get_cell().tolist())

    if atoms.has('tags'):
        b.write(tags=atoms.get_tags())
    if atoms.has('momenta'):
        b.write(momenta=atoms.get_momenta())
    if atoms.has('magmoms'):
        b.write(magmoms=atoms.get_initial_magnetic_moments())
    if atoms.has('charges'):
        b.write(charges=atoms.get_initial_charges())


def read_traj(filename, index):
    trj = TrajectoryReader(filename)
    for i in range(*index.indices(len(trj))):
        yield trj[i]


def write_traj(filename, images):
    """Write image(s) to trajectory."""
    trj = TrajectoryWriter(filename, mode='w')
    if isinstance(images, Atoms):
        images = [images]
    for atoms in images:
        trj.write(atoms)
    trj.close()


class OldCalculatorWrapper:
    def __init__(self, calc):
        self.calc = calc
        try:
            self.name = calc.name
        except AttributeError:
            self.name = calc.__class__.__name__.lower()

    def get_property(self, prop, atoms, allow_calculation=True):
        try:
            if (not allow_calculation and
                self.calc.calculation_required(atoms, [prop])):
                return None
        except AttributeError:
            pass

        method = 'get_' + {'energy': 'potential_energy',
                           'magmom': 'magnetic_moment',
                           'magmoms': 'magnetic_moments',
                           'dipole': 'dipole_moment'}.get(prop, prop)
        try:
            result = getattr(self.calc, method)(atoms)
        except AttributeError:
            raise PropertyNotImplementedError
        return result


def convert(name):
    import os
    t = TrajectoryWriter(name + '.new')
    for atoms in PickleTrajectory(name, _warn=False):
        t.write(atoms)
    t.close()
    os.rename(name, name + '.old')
    os.rename(name + '.new', name)


def main():
    import optparse
    parser = optparse.OptionParser(usage='python -m ase.io.trajectory '
                                   'a1.traj [a2.traj ...]',
                                   description='Convert old trajectory '
                                   'file(s) to new format. '
                                   'The old file is kept as a1.traj.old.')
    opts, args = parser.parse_args()
    for name in args:
        convert(name)


if __name__ == '__main__':
    main()



from ase.build import molecule
from ase.constraints import FixAtoms

N = 2

atoms = molecule('CO2')
atoms.set_cell((15, 15, 15))

# Indices method:
atomsi = atoms.copy()
atomsi.set_constraint(FixAtoms(indices=[0]))
atomsi = atomsi.repeat((N, 1, 1))

atomsiref = atoms.copy().repeat((N, 1, 1))
atomsiref.set_constraint(FixAtoms(indices=list(range(0, 3 * N, 3))))

lcatomsi = list(atomsi.constraints[0].index)
lcatomsiref = list(atomsiref.constraints[0].index)

assert lcatomsi == lcatomsiref

# Mask method:
atomsm = atoms.copy()
atomsm.set_constraint(FixAtoms(mask=[True, False, False]))
atomsm = atomsm.repeat((N, 1, 1))

atomsmref = atoms.copy().repeat((N, 1, 1))
atomsmref.set_constraint(FixAtoms(mask=[True, False, False] * N))

lcatomsm = list(atomsm.constraints[0].index)
lcatomsmref = list(atomsmref.constraints[0].index)

assert lcatomsm == lcatomsmref
assert lcatomsm == lcatomsi



from ase import Atoms

# Setup a chain of H,O,C
# H-O Dist = 2
# O-C Dist = 3
# C-H Dist = 5 with mic=False
# C-H Dist = 4 with mic=True
a = Atoms('HOC', positions=[(1, 1, 1), (3, 1, 1), (6, 1, 1)])
a.set_cell((9, 2, 2))
a.set_pbc((True, False, False))


# Calculate indiviually with mic=True
assert a.get_distance(0, 1, mic=True) == 2
assert a.get_distance(1, 2, mic=True) == 3
assert a.get_distance(0, 2, mic=True) == 4

# Calculate indiviually with mic=False
assert a.get_distance(0, 1, mic=False) == 2
assert a.get_distance(1, 2, mic=False) == 3
assert a.get_distance(0, 2, mic=False) == 5

# Calculate in groups with mic=True
assert (a.get_distances(0, [1, 2], mic=True) == [2, 4]).all()

# Calculate in groups with mic=False
assert (a.get_distances(0, [1, 2], mic=False) == [2, 5]).all()

# Calculate all with mic=True
assert (a.get_all_distances(mic=True) == [[0, 2, 4],
                                          [2, 0, 3],
                                          [4, 3, 0]]).all()

# Calculate all with mic=False
assert (a.get_all_distances(mic=False) == [[0, 2, 5],
                                           [2, 0, 3],
                                           [5, 3, 0]]).all()



# -*- coding: utf-8 -*-
"""colors.py - select how to color the atoms in the GUI."""
from __future__ import unicode_literals
from ase.gui.i18n import _

import numpy as np

import ase.gui.ui as ui


class ColorWindow:
    """A window for selecting how to color the atoms."""
    def __init__(self, gui):
        self.win = ui.Window(_('Colors'))
        self.gui = gui
        self.win.add(ui.Label(_('Choose how the atoms are colored:')))
        values = ['jmol', 'tag', 'force', 'velocity', 'charge', 'magmom']
        labels = [_('By atomic number, default "jmol" colors'),
                  _('By tag'),
                  _('By force'),
                  _('By velocity'),
                  _('By charge'),
                  _('By magnetic moment')]

        self.radio = ui.RadioButtons(labels, values, self.toggle,
                                     vertical=True)
        self.radio.value = gui.colormode
        self.win.add(self.radio)
        self.activate()
        self.label = ui.Label()
        self.win.add(self.label)

    def activate(self):
        images = self.gui.images
        radio = self.radio
        radio['tag'].active = images.T.any()
        radio['force'].active = np.isfinite(images.F).all()
        radio['velocity'].active = np.isfinite(images.V).all()
        radio['charge'].active = images.q.any()
        radio['magmom'].active = images.M.any()

    def toggle(self, value):
        if value == 'jmol':
            self.gui.set_colors()
            text = ''
        else:
            self.gui.colormode = value
            scalars = np.array([self.gui.get_color_scalars(i)
                                for i in range(len(self.gui.images))])
            mn = scalars.min()
            mx = scalars.max()
            colorscale = ['#{0:02X}AA00'.format(red)
                          for red in range(0, 240, 10)]
            self.gui.colormode_data = colorscale, mn, mx

            unit = {'tag': '',
                    'force': 'eV/Ang',
                    'velocity': '??',
                    'charge': '|e|',
                    u'magmom': 'μB'}[value]
            text = '[{0},{1}]: [{2:.6f},{3:.6f}] {4}'.format(
                _('Green'), _('Yellow'), mn, mx, unit)

        self.label.text = text
        self.radio.value = value
        self.gui.draw()
        return text  # for testing

    def notify_atoms_changed(self):
        "Called by gui object when the atoms have changed."
        self.activate()
        mode = self.gui.colormode
        if not self.radio[mode].active:
            mode = 'jmol'
        self.toggle(mode)



""" test run for DFTB+ calculator
    the tolerance is extremely loose, beause different sk files
    give different results

"""
import os

from ase import Atoms
from ase.calculators.dftb import Dftb
from ase.constraints import FixAtoms
from ase.optimize import QuasiNewton
from ase.build import fcc111, add_adsorbate

h = 1.85
d = 1.10

slab = fcc111('Ni', size=(2, 2, 3), vacuum=10.0)
calc1 = Dftb(label='slab',
             kpts=[2, 2, 1],
             Hamiltonian_MaxAngularMomentum_='',
             Hamiltonian_MaxAngularMomentum_Ni='"d"',
             Hamiltonian_SCC='YES')
slab.set_calculator(calc1)
dyn = QuasiNewton(slab, trajectory='slab.traj')
dyn.run(fmax=0.05)
e_slab = slab.get_potential_energy()
os.system('rm dftb_in.hsd')
molecule = Atoms('2N', positions=[(0., 0., 0.), (0., 0., d)])
calc2 = Dftb(label='n2',
             Hamiltonian_MaxAngularMomentum_='',
             Hamiltonian_MaxAngularMomentum_N='"p"',
             Hamiltonian_SCC='YES')
molecule.set_calculator(calc2)
dyn = QuasiNewton(molecule, trajectory='n2.traj')
dyn.run(fmax=0.05)
e_N2 = molecule.get_potential_energy()

slab2 = slab
add_adsorbate(slab2, molecule, h, 'ontop')
constraint = FixAtoms(mask=[a.symbol != 'N' for a in slab2])
slab2.set_constraint(constraint)
calc3 = Dftb(label='slab2',
             kpts=[2, 2, 1],
             Hamiltonian_MaxAngularMomentum_='',
             Hamiltonian_MaxAngularMomentum_N='"p"',
             Hamiltonian_MaxAngularMomentum_Ni='"d"',
             Hamiltonian_SCC='YES')
slab2.set_calculator(calc3)
dyn = QuasiNewton(slab2, trajectory='slab2.traj')
dyn.run(fmax=0.05)

adsorption_energy = e_slab + e_N2 - slab2.get_potential_energy()
assert abs(adsorption_energy + 0.4227415) < 0.3
files = ['band.out', 'detailed.out', 'dftb_in.hsd', 'dftb_pin.hsd',
         'geo_end.gen', 'geo_end.xyz', 'results.tag',
         'n2.traj', 'n2.traj.bak', 'n2.out',
         'slab.traj', 'slab.traj.bak', 'slab.out',
         'slab2.traj', 'slab2.traj.bak', 'slab2.out']



"""Make sure we can't read old traj file, but we can convert them."""
import ase.io as aio
from ase import Atoms
from ase.io.pickletrajectory import PickleTrajectory
from ase.io.trajectory import convert
from ase.test import must_raise


t = PickleTrajectory('hmm.traj', 'w', _warn=False)
a = Atoms('H')
t.write(a)
t.close()

with must_raise(DeprecationWarning):
    aio.read('hmm.traj')
    
convert('hmm.traj')
aio.read('hmm.traj')



from ase.units import Bohr


def read_turbomole(f='coord'):
    """Method to read turbomole coord file
    
    coords in bohr, atom types in lowercase, format:
    $coord
    x y z atomtype
    x y z atomtype f
    $end
    Above 'f' means a fixed atom.
    """
    from ase import Atoms
    from ase.constraints import FixAtoms

    lines = f.readlines()
    atoms_pos = []
    atom_symbols = []
    myconstraints=[]
    
    # find $coord section;
    # does not necessarily have to be the first $<something> in file...
    for i, l in enumerate(lines):
        if l.strip().startswith('$coord'):
            start = i
            break
    for line in lines[start+1:]:
        if line.startswith('$'): # start of new section
            break
        else:
            x, y, z, symbolraw = line.split()[:4]
            symbolshort=symbolraw.strip()
            symbol=symbolshort[0].upper()+symbolshort[1:].lower()
            #print symbol
            atom_symbols.append(symbol)
            atoms_pos.append([float(x)*Bohr, float(y)*Bohr, float(z)*Bohr])
            cols = line.split()
            if (len(cols) == 5):
                fixedstr = line.split()[4].strip()
                if (fixedstr == "f"):
                    myconstraints.append(True)
                else:
                    myconstraints.append(False)
            else:
                myconstraints.append(False)
            
    atoms = Atoms(positions = atoms_pos, symbols = atom_symbols, pbc = False)
    c = FixAtoms(mask = myconstraints)
    atoms.set_constraint(c)
    return atoms

    
def read_turbomole_gradient(f='gradient', index=-1):
    """ Method to read turbomole gradient file """

    # read entire file
    lines = [x.strip() for x in f.readlines()]

    # find $grad section
    start = end = -1
    for i, line in enumerate(lines):
        if not line.startswith('$'):
            continue
        if line.split()[0] == '$grad':
            start = i
        elif start >= 0:
            end = i
            break

    if end <= start:
        raise RuntimeError('File does not contain a valid \'$grad\' section')

    def formatError():
        raise RuntimeError('Data format in file does not correspond to known '
                           'Turbomole gradient format')

    # trim lines to $grad
    del lines[:start+1]
    del lines[end-1-start:]

    # Interpret $grad section
    from ase import Atoms, Atom
    from ase.calculators.singlepoint import SinglePointCalculator
    from ase.units import Bohr, Hartree
    images = []
    while len(lines): # loop over optimization cycles
        # header line
        # cycle =      1    SCF energy =     -267.6666811409   |dE/dxyz| =  0.157112
        fields = lines[0].split('=')
        try:
            # cycle = int(fields[1].split()[0])
            energy = float(fields[2].split()[0]) * Hartree
            # gradient = float(fields[3].split()[0])
        except (IndexError, ValueError):
            formatError()
        
        # coordinates/gradient
        atoms = Atoms()
        forces = []
        for line in lines[1:]:
            fields = line.split()
            if len(fields) == 4: # coordinates
                # 0.00000000000000      0.00000000000000      0.00000000000000      c
                try:
                    symbol = fields[3].lower().capitalize()
                    position = tuple([Bohr * float(x) for x in fields[0:3] ])
                except ValueError:
                    formatError()
                atoms.append(Atom(symbol, position))
            elif len(fields) == 3: # gradients
                #  -.51654903354681D-07  -.51654903206651D-07  0.51654903169644D-07
                try:
                    grad = [-float(x.replace('D', 'E')) * Hartree / Bohr for x in fields[0:3] ]
                except ValueError:
                    formatError()
                forces.append(grad)
            else: # next cycle
                break

        # calculator
        calc = SinglePointCalculator(atoms, energy=energy, forces=forces)
        atoms.set_calculator(calc)

        # save frame
        images.append(atoms)

        # delete this frame from data to be handled
        del lines[:2*len(atoms)+1]

    return images[index]


def write_turbomole(filename, atoms):
    """Method to write turbomole coord file
    """

    import numpy as np
    from ase.constraints import FixAtoms

    f = filename

    coord = atoms.get_positions()
    symbols = atoms.get_chemical_symbols()

    fix_index = {'indices': [], 'mask': np.zeros(len(symbols),dtype=bool)}
    if atoms.constraints:
        for constr in atoms.constraints:
            if isinstance(constr, FixAtoms):
                if 'indices' in constr.todict().keys():
                    fix_index['indices'].extend(constr.todict()['indices'])
                if 'mask' in constr.todict().keys():
                    fix_index['mask'] += constr.todict()['mask']

    fix_index['indices'] = np.unique(fix_index['indices'])
    fix_index['mask'] = np.transpose(np.nonzero(fix_index['mask']))

    fix_str = []
    for i in range(len(atoms)):
        if i in fix_index['mask'] or i in fix_index['indices']:
            fix_str.append('f')
        else:
            fix_str.append('')

    f.write('$coord\n')
    for (x, y, z), s, fix in zip(coord, symbols, fix_str):
        f.write('%20.14f  %20.14f  %20.14f      %2s  %2s \n'
                % (x / Bohr, y / Bohr, z / Bohr, s.lower(), fix))

    f.write('$end\n')



import numpy as np

from ase.old import OldASEListOfAtomsWrapper

try:
    import Numeric as num
except ImportError:
    pass

def np2num(a, typecode=None):
    if num.__version__ > '23.8':
        return num.array(a, typecode)
    if typecode is None:
        typecode = num.Float
    b = num.fromstring(a.tostring(), typecode)
    b.shape = a.shape
    return b

def restart(filename, **kwargs):
    calc = Dacapo(filename, **kwargs)
    atoms = calc.get_atoms()
    return atoms, calc

class Dacapo:
    def __init__(self, filename=None, stay_alive=False, stress=False,
                 **kwargs):

        self.kwargs = kwargs
        self.stay_alive = stay_alive
        self.stress = stress
        
        if filename is not None:
            from Dacapo import Dacapo
            self.loa = Dacapo.ReadAtoms(filename, **kwargs)
            self.calc = self.loa.GetCalculator()
        else:
            self.loa = None
            self.calc = None

        self.pps = []
        
    def set_pp(self, Z, path):
        self.pps.append((Z, path))

    def set_txt(self, txt):
        if self.calc is None:
            self.kwargs['txtout'] = txt
        else:
            self.calc.SetTxtFile(txt)

    def set_nc(self, nc):
        if self.calc is None:
            self.kwargs['out'] = nc
        else:
            self.calc.SetNetCDFFile(nc)

    def update(self, atoms):
        from Dacapo import Dacapo
        if self.calc is None:
            if 'nbands' not in self.kwargs:
                n = sum([valence[atom.symbol] for atom in atoms])
                self.kwargs['nbands'] = int(n * 0.65) + 4

            magmoms = atoms.get_initial_magnetic_moments()
            if magmoms.any():
                self.kwargs['spinpol'] = True

            self.calc = Dacapo(**self.kwargs)

            if self.stay_alive:
                self.calc.StayAliveOn()
            else:
                self.calc.StayAliveOff()

            if self.stress:
                self.calc.CalculateStress()

            for Z, path in self.pps:
                self.calc.SetPseudoPotential(Z, path)

        if self.loa is None:
            from ASE import Atom, ListOfAtoms
            numbers = atoms.get_atomic_numbers()
            positions = atoms.get_positions()
            magmoms = atoms.get_initial_magnetic_moments()
            self.loa = ListOfAtoms([Atom(Z=numbers[a],
                                         position=positions[a],
                                         magmom=magmoms[a])
                                    for a in range(len(atoms))],
                                   cell=np2num(atoms.get_cell()),
                                   periodic=tuple(atoms.get_pbc()))
            self.loa.SetCalculator(self.calc)
        else:
            self.loa.SetCartesianPositions(np2num(atoms.get_positions()))
            self.loa.SetUnitCell(np2num(atoms.get_cell()), fix=True)
            
    def get_atoms(self):
        atoms = OldASEListOfAtomsWrapper(self.loa).copy()
        atoms.set_calculator(self)
        return atoms
    
    def get_potential_energy(self, atoms):
        self.update(atoms)
        return self.calc.GetPotentialEnergy()

    def get_forces(self, atoms):
        self.update(atoms)
        return np.array(self.calc.GetCartesianForces())

    def get_stress(self, atoms):
        self.update(atoms)
        stress = np.array(self.calc.GetStress())
        if stress.ndim == 2:
            return stress.ravel()[[0, 4, 8, 5, 2, 1]]
        else:
            return stress

    def calculation_required(self, atoms, quantities):
        if self.calc is None:
            return True

        if atoms != self.get_atoms():
            return True

        return False
        
    def get_number_of_bands(self):
        return self.calc.GetNumberOfBands()

    def get_k_point_weights(self):
        return np.array(self.calc.GetIBZKPointWeights())

    def get_number_of_spins(self):
        return 1 + int(self.calc.GetSpinPolarized())

    def get_eigenvalues(self, kpt=0, spin=0):
        return np.array(self.calc.GetEigenvalues(kpt, spin))

    def get_fermi_level(self):
        return self.calc.GetFermiLevel()

    def get_magnetic_moment(self):
        return self.calc.GetMagneticMoment()

    def get_number_of_electrons(self):
        return self.calc.GetValenceElectrons()

    def get_number_of_grid_points(self):
        return np.array(self.get_pseudo_wave_function(0, 0, 0).shape)

    def get_pseudo_density(self, spin=0):
        return np.array(self.calc.GetDensityArray(spin))
    
    def get_pseudo_wave_function(self, band=0, kpt=0, spin=0, pad=True):
        kpt_c = self.get_bz_k_points()[kpt]
        state = self.calc.GetElectronicStates().GetState(band=band, spin=spin,
                                                         kptindex=kpt)

        # Get wf, without bloch phase (Phase = True doesn't do anything!)
        wave = state.GetWavefunctionOnGrid(phase=False)

        # Add bloch phase if this is not the Gamma point
        if np.all(kpt_c == 0):
            return wave
        coord = state.GetCoordinates()
        phase = coord[0] * kpt_c[0] + coord[1] * kpt_c[1] + coord[2] * kpt_c[2]
        return np.array(wave) * np.exp(-2.j * np.pi * phase) # sign! XXX

        #return np.array(self.calc.GetWaveFunctionArray(n, k, s)) # No phase!

    def get_bz_k_points(self):
        return np.array(self.calc.GetBZKPoints())

    def get_ibz_k_points(self):
        return np.array(self.calc.GetIBZKPoints())

    def get_wannier_localization_matrix(self, nbands, dirG, kpoint,
                                        nextkpoint, G_I, spin):
        return np.array(self.calc.GetWannierLocalizationMatrix(
            G_I=G_I.tolist(), nbands=nbands, dirG=dirG.tolist(),
            kpoint=kpoint, nextkpoint=nextkpoint, spin=spin))
    
    def initial_wannier(self, initialwannier, kpointgrid, fixedstates,
                        edf, spin):
        # Use initial guess to determine U and C
        init = self.calc.InitialWannier(initialwannier, self.atoms,
                                        np2num(kpointgrid, num.Int))

        states = self.calc.GetElectronicStates()
        waves = [[state.GetWaveFunction()
                  for state in states.GetStatesKPoint(k, spin)]
                 for k in self.calc.GetIBZKPoints()]

        init.SetupMMatrix(waves, self.calc.GetBZKPoints())
        c, U = init.GetListOfCoefficientsAndRotationMatrices(
            (self.calc.GetNumberOfBands(), fixedstates, edf))
        U = np.array(U)
        for k in range(len(c)):
            c[k] = np.array(c[k])
        return c, U

valence = {
'H':   1,
'B':   3,
'C':   4,
'N':   5,
'O':   6,
'Li':  1,
'Na':  1,
'K':   9,
'Mg':  8,
'Ca': 10,
'Sr': 10,
'Al':  3,
'Ga': 13,
'Sc': 11,
'Ti': 12,
'V':  13,
'Cr': 14,
'Mn':  7,
'Fe':  8,
'Co':  9,
'Ni': 10,
'Cu': 11,
'Zn': 12,
'Y':  11,
'Zr': 12,
'Nb': 13,
'Mo':  6,
'Ru':  8,
'Rh':  9,
'Pd': 10,
'Ag': 11,
'Cd': 12,
'Ir': 9,
'Pt': 10,
'Au': 11,
}



"""WSGI Flask-app for browsing a database.

You can launch Flask's local webserver like this::

    $ ase-db abc.db -w

For a real webserver, you need to set the $ASE_DB_APP_CONFIG environment
variable to point to a configuration file like this::

    ASE_DB_NAME = '/path/to/db-file/abc.db'
    ASE_DB_HOMEPAGE = '<a href="https://home.page.dk">HOME</a> ::'

Start with something like::

    twistd web --wsgi=ase.db.app.app --port=8000

"""

import collections
import functools
import io
import os
import re
import tempfile

from flask import Flask, render_template, request, send_from_directory

try:
    import matplotlib
    matplotlib.use('Agg', warn=False)
except ImportError:
    pass

import ase.db
from ase.db.plot import atoms2png, dct2plot
from ase.db.summary import Summary
from ase.db.table import Table, all_columns
from ase.visualize import view


# Every client-connetions gets one of these tuples:
Connection = collections.namedtuple(
    'Connection',
    ['query',  # query string
     'nrows',  # number of rows matched
     'page',  # page number
     'columns',  # what columns to show
     'sort',  # what column to sort after
     'limit',  # number of rows per page
     'opened'  # list of id's in the table that are open
     ])

app = Flask(__name__)

db = None
home = ''  # link to homepage
open_ase_gui = True  # click image to open ase-gui

if 'ASE_DB_APP_CONFIG' in os.environ:
    app.config.from_envvar('ASE_DB_APP_CONFIG')
    db = ase.db.connect(app.config['ASE_DB_NAME'])
    home = app.config['ASE_DB_HOMEPAGE']
    open_ase_gui = False

next_con_id = 1
connections = {}
tmpdir = tempfile.mkdtemp()  # used to cache png-files

# Find numbers in formulas so that we can convert H2O to H<sub>2</sub>O:
SUBSCRIPT = re.compile(r'(\d+)')


@app.route('/')
def index():
    global next_con_id
    con_id = int(request.args.get('x', '0'))
    if con_id not in connections:
        con_id = next_con_id
        next_con_id += 1
        query = ''
        columns = list(all_columns)
        sort = 'id'
        limit = 25
        opened = set()
        nrows = None
        page = 0
    else:
        query, nrows, page, columns, sort, limit, opened = connections[con_id]

    if 'sort' in request.args:
        column = request.args['sort']
        if column == sort:
            sort = '-' + column
        elif '-' + column == sort:
            sort = 'id'
        else:
            sort = column
        page = 0
    elif 'query' in request.args:
        query = request.args['query'].encode()
        try:
            limit = max(1, min(int(request.args.get('limit', limit)), 200))
        except ValueError:
            pass
        sort = 'id'
        opened = set()
        page = 0
        nrows = None
    elif 'page' in request.args:
        page = int(request.args['page'])

    if 'toggle' in request.args:
        tcolumns = request.args['toggle'].split(',')
        if tcolumns == ['reset']:
            columns = list(all_columns)
        else:
            for column in tcolumns:
                if column in columns:
                    columns.remove(column)
                    if column == sort.lstrip('-'):
                        sort = 'id'
                        page = 0
                else:
                    columns.append(column)

    if nrows is None:
        nrows = db.count(query)

    table = Table(db)
    table.select(query, columns, sort, limit, offset=page * limit)
    con = Connection(query, nrows, page, columns, sort, limit, opened)
    connections[con_id] = con
    table.format(SUBSCRIPT)
    addcolumns = [column for column in all_columns + table.keys
                  if column not in table.columns]

    return render_template('table.html', t=table, con=con, cid=con_id,
                           home=home, pages=pages(page, nrows, limit),
                           nrows=nrows,
                           addcolumns=addcolumns,
                           row1=page * limit + 1,
                           row2=min((page + 1) * limit, nrows))


@app.route('/open_row/<int:id>')
def open_row(id):
    con_id = int(request.args['x'])
    opened = connections[con_id].opened
    if id in opened:
        opened.remove(id)
        return ''
    opened.add(id)
    return render_template('more.html',
                           dct=db.get(id), id=id, cid=con_id)


@app.route('/image/<name>')
def image(name):
    path = os.path.join(tmpdir, name)
    if not os.path.isfile(path):
        id = int(name[:-4])
        atoms = db.get_atoms(id)
        atoms2png(atoms, path)

    return send_from_directory(tmpdir, name)


@app.route('/cif/<name>')
def cif(name):
    path = os.path.join(tmpdir, name)
    if not os.path.isfile(path):
        id = int(name[:-4])
        atoms = db.get_atoms(id)
        atoms.write(path)
    return send_from_directory(tmpdir, name)


@app.route('/plot/<png>')
def plot(png):
    path = os.path.join(tmpdir, png)
    if not os.path.isfile(path):
        name, id = png[:-4].split('-')
        dct = db[int(id)].data
        dct2plot(dct, name, path, show=False)

    return send_from_directory(tmpdir, png)


@app.route('/gui/<int:id>')
def gui(id):
    if open_ase_gui:
        atoms = db.get_atoms(id)
        view(atoms)
    return '', 204, []


@app.route('/id/<int:id>')
def summary(id):
    s = Summary(db.get(id), SUBSCRIPT)
    return render_template('summary.html', s=s, home=home,
                           open_ase_gui=open_ase_gui)


def tofile(query, type, limit=0):
    fd, name = tempfile.mkstemp(suffix='.' + type)
    con = ase.db.connect(name, use_lock_file=False)
    for dct in db.select(query, limit=limit):
        con.write(dct,
                  data=dct.get('data', {}),
                  **dct.get('key_value_pairs', {}))
    os.close(fd)
    data = open(name).read()
    os.unlink(name)
    return data


def download(f):
    @functools.wraps(f)
    def ff(*args, **kwargs):
        text, name = f(*args, **kwargs)
        headers = [('Content-Disposition',
                    'attachment; filename="{0}"'.format(name)),
                   ]  # ('Content-type', 'application/sqlite3')]
        return text, 200, headers
    return ff


@app.route('/xyz/<int:id>')
@download
def xyz(id):
    fd = io.BytesIO()
    from ase.io.xyz import write_xyz
    write_xyz(fd, db.get_atoms(id))
    data = fd.getvalue()
    return data, '{0}.xyz'.format(id)


@app.route('/json')
@download
def jsonall():
    con_id = int(request.args['x'])
    con = connections[con_id]
    data = tofile(con.query, 'json', con.limit)
    return data, 'selection.json'


@app.route('/json/<int:id>')
@download
def json(id):
    data = tofile(id, 'json')
    return data, '{0}.json'.format(id)


@app.route('/sqlite')
@download
def sqliteall():
    con_id = int(request.args['x'])
    con = connections[con_id]
    data = tofile(con.query, 'db', con.limit)
    return data, 'selection.db'


@app.route('/sqlite/<int:id>')
@download
def sqlite(id):
    data = tofile(id, 'db')
    return data, '{0}.db'.format(id)


@app.route('/robots.txt')
def robots():
    return 'User-agent: *\nDisallow: /\n', 200


def pages(page, nrows, limit):
    npages = (nrows + limit - 1) // limit
    p1 = min(5, npages)
    p2 = max(page - 4, p1)
    p3 = min(page + 5, npages)
    p4 = max(npages - 4, p3)
    pgs = list(range(p1))
    if p1 < p2:
        pgs.append(-1)
    pgs += list(range(p2, p3))
    if p3 < p4:
        pgs.append(-1)
    pgs += list(range(p4, npages))
    pages = [(page - 1, 'previous')]
    for p in pgs:
        if p == -1:
            pages.append((-1, '...'))
        elif p == page:
            pages.append((-1, str(p + 1)))
        else:
            pages.append((p, str(p + 1)))
    nxt = min(page + 1, npages - 1)
    if nxt == page:
        nxt = -1
    pages.append((nxt, 'next'))
    return pages
